#!/usr/bin/python

#
# Copyright (c) 2021 by VMware, Inc. ("VMware")
# Used Copyright (c) 2018 by Network Device Education Foundation,
# Inc. ("NetDEF") in this file.
#
# Permission to use, copy, modify, and/or distribute this software
# for any purpose with or without fee is hereby granted, provided
# that the above copyright notice and this permission notice appear
# in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND VMWARE DISCLAIMS ALL WARRANTIES
# WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL VMWARE BE LIABLE FOR
# ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY
# DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
# WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS
# ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE
# OF THIS SOFTWARE.
#
"""

1. Verify mgmt commit check.
2. Verify mgmt commit apply.
3. Verify mgmt commit abort.
4. Verify mgmt delete config.
5. Kill mgmtd - verify that static routes are intact.
6. Kill mgmtd - verify that watch frr restarts.
7. Show and CLI - Execute all the newly introduced commands of mgmtd.
8. Verify mgmt rollback functionality.

"""
import sys
import time
import os
import pytest
import platform
import json

# Save the Current Working Directory to find configuration files.
CWD = os.path.dirname(os.path.realpath(__file__))
sys.path.append(os.path.join(CWD, "../"))
sys.path.append(os.path.join(CWD, "../lib/"))

# pylint: disable=C0413
# Import topogen and topotest helpers
from lib.topogen import Topogen, get_topogen
from lib.topotest import version_cmp, router_json_cmp

# Import topoJson from lib, to create topology and initial configuration
from lib.common_config import (
    start_topology,
    write_test_header,
    write_test_footer,
    reset_config_on_routers,
    verify_rib,
    create_static_routes,
    check_address_types,
    step,
    shutdown_bringup_interface,
    stop_router,
    start_router,
    apply_raw_config,
    kill_router_daemons,
    start_router_daemons,
)
from lib.topolog import logger
from lib.topojson import build_config_from_json

pytestmark = [pytest.mark.bgpd, pytest.mark.staticd, pytest.mark.mgmtd]

# Global variables
ADDR_TYPES = check_address_types()
NETWORK = {"ipv4": ["11.0.20.1/32", "11.0.20.2/32"], "ipv6": ["2::1/128", "2::2/128"]}
NETWORK2 = {"ipv4": "11.0.20.1/32", "ipv6": "2::1/128"}
PREFIX1 = {"ipv4": "110.0.20.1/32", "ipv6": "20::1/128"}


def setup_module(mod):
    """
    Sets up the pytest environment.

    * `mod`: module name
    """

    testsuite_run_time = time.asctime(time.localtime(time.time()))
    logger.info("Testsuite start time: {}".format(testsuite_run_time))
    logger.info("=" * 40)

    logger.info("Running setup_module to create topology")

    # This function initiates the topology build with Topogen...
    json_file = "{}/yang_mgmt.json".format(CWD)
    tgen = Topogen(json_file, mod.__name__)
    global topo
    topo = tgen.json_topo
    # ... and here it calls Mininet initialization functions.

    # Starting topology, create tmp files which are loaded to routers
    #  to start deamons and then start routers
    start_topology(tgen)

    # Creating configuration from JSON
    build_config_from_json(tgen, topo)

    if version_cmp(platform.release(), "4.19") < 0:
        error_msg = (
            'These tests will not run. (have kernel "{}", '
            "requires kernel >= 4.19)".format(platform.release())
        )
        pytest.skip(error_msg)

    # Don't run this test if we have any failure.
    if tgen.routers_have_failure():
        pytest.skip(tgen.errors)

    logger.info("Running setup_module() done")


def teardown_module(mod):
    """
    Teardown the pytest environment.

    * `mod`: module name
    """

    logger.info("Running teardown_module to delete topology: %s", mod)

    tgen = get_topogen()

    # Stop toplogy and Remove tmp files
    tgen.stop_topology()

    logger.info(
        "Testsuite end time: {}".format(time.asctime(time.localtime(time.time())))
    )
    logger.info("=" * 40)


def populate_nh():
    """
    Populate nexthops.
    """

    next_hop_ip = {
        "nh1": {
            "ipv4": topo["routers"]["r1"]["links"]["r2-link0"]["ipv4"].split("/")[0],
            "ipv6": topo["routers"]["r1"]["links"]["r2-link0"]["ipv6"].split("/")[0],
        },
        "nh2": {
            "ipv4": topo["routers"]["r1"]["links"]["r2-link1"]["ipv4"].split("/")[0],
            "ipv6": topo["routers"]["r1"]["links"]["r2-link1"]["ipv6"].split("/")[0],
        },
    }
    return next_hop_ip


#####################################################
#
#   Testcases
#
#####################################################


def test_mgmt_commit_check(request):
    """
    Verify mgmt commit check.

    """
    tc_name = request.node.name
    write_test_header(tc_name)
    tgen = get_topogen()
    # Don't run this test if we have any failure.
    if tgen.routers_have_failure():
        pytest.skip(tgen.errors)

    reset_config_on_routers(tgen)

    step("Mgmt Commit check")
    raw_config = {
        "r1": {
            "raw_config": [
                "mgmt set-config /frr-routing:routing/control-plane-protocols/control-plane-protocol[type='frr-staticd:staticd'][name='staticd'][vrf='default']/frr-staticd:staticd/route-list[prefix='192.1.1.2/32'][src-prefix='::/0'][afi-safi='frr-routing:ipv4-unicast']/path-list[table-id='0'][distance='1']/frr-nexthops/nexthop[nh-type='blackhole'][vrf='default'][gateway=''][interface='(null)']/bh-type unspec",
                "mgmt commit check",
            ]
        }
    }

    result = apply_raw_config(tgen, raw_config)
    assert result is True, "Testcase {} : Failed Error: {}".format(tc_name, result)

    step("Mgmt Commit check")
    raw_config = {
        "r1": {
            "raw_config": [
                "mgmt set-config /frr-routing:routing/control-plane-protocols/control-plane-protocol[type='frr-staticd:staticd'][name='staticd'][vrf='default']/frr-staticd:staticd/route-list[prefix='192.1.1.2/32'][src-prefix='::/0'][afi-safi='frr-routing:ipv4-unicast']/path-list[table-id='0'][distance='1']/frr-nexthops/nexthop[nh-type='blackhole'][vrf='default'][gateway=''][interface='(null)']/bh-type unspec",
                "mgmt commit check",
            ]
        }
    }

    result = apply_raw_config(tgen, raw_config)
    assert result is True, "Testcase {} : Failed Error: {}".format(tc_name, result)

    step("verify that the route is not configured, as commit apply not done.")

    dut = "r1"
    protocol = "static"
    input_dict_4 = {
        "r2": {
            "static_routes": [
                {
                    "network": "1192.1.1.2/32",
                    "next_hop": "Null0",
                }
            ]
        }
    }
    result = verify_rib(
        tgen, "ipv4", dut, input_dict_4, protocol=protocol, expected=False
    )
    assert (
        result is not True
    ), "Testcase {} : Failed" "Error: Routes is missing in RIB".format(tc_name)

    write_test_footer(tc_name)


def test_mgmt_commit_apply(request):
    """
    Verify mgmt commit apply.

    """
    tc_name = request.node.name
    write_test_header(tc_name)
    tgen = get_topogen()
    # Don't run this test if we have any failure.
    if tgen.routers_have_failure():
        pytest.skip(tgen.errors)

    reset_config_on_routers(tgen)

    step("Mgmt Commit apply with Valid Configuration")
    raw_config = {
        "r1": {
            "raw_config": [
                "mgmt set-config /frr-routing:routing/control-plane-protocols/control-plane-protocol[type='frr-staticd:staticd'][name='staticd'][vrf='default']/frr-staticd:staticd/route-list[prefix='192.1.1.20/32'][src-prefix='::/0'][afi-safi='frr-routing:ipv4-unicast']/path-list[table-id='0'][distance='1']/frr-nexthops/nexthop[nh-type='blackhole'][vrf='default'][gateway=''][interface='(null)']/vrf default",
                "mgmt commit apply",
            ]
        }
    }

    result = apply_raw_config(tgen, raw_config)
    assert result is True, "Testcase {} : Failed Error: {}".format(tc_name, result)

    step("Mgmt Commit apply with Invalid Configuration")
    raw_config = {
        "r1": {
            "raw_config": [
                "mgmt set-config /frr-routing:routing/control-plane-protocols/control-plane-protocol[type='frr-staticd:staticd'][name='staticd'][vrf='default']/frr-staticd:staticd/route-list[prefix='192.1.1.20/32'][src-prefix='::/0'][afi-safi='frr-routing:ipv4-unicast']/path-list[table-id='0'][distance='1']/frr-nexthops/nexthop[nh-type='blackhole'][vrf='default'][gateway=''][interface='(null)']/vrf default",
                "mgmt commit apply",
            ]
        }
    }

    result = apply_raw_config(tgen, raw_config)
    assert result is not True, "Testcase {} : Failed Error: {}".format(tc_name, result)

    step("verify that the route is configured")

    dut = "r1"
    protocol = "static"
    input_dict_4 = {"r2": {"static_routes": [{"network": "192.1.1.20/32"}]}}
    result = verify_rib(tgen, "ipv4", dut, input_dict_4, protocol=protocol)
    assert (
        result is True
    ), "Testcase {} : Failed" "Error: Routes is missing in RIB".format(tc_name)

    write_test_footer(tc_name)


def test_mgmt_commit_abort(request):
    """
    Verify mgmt commit abort.

    """
    tc_name = request.node.name
    write_test_header(tc_name)
    tgen = get_topogen()
    # Don't run this test if we have any failure.
    if tgen.routers_have_failure():
        pytest.skip(tgen.errors)

    reset_config_on_routers(tgen)

    step("Mgmt Commit abort")
    raw_config = {
        "r1": {
            "raw_config": [
                "mgmt set-config /frr-routing:routing/control-plane-protocols/control-plane-protocol[type='frr-staticd:staticd'][name='staticd'][vrf='default']/frr-staticd:staticd/route-list[prefix='192.1.1.3/32'][src-prefix='::/0'][afi-safi='frr-routing:ipv4-unicast']/path-list[table-id='0'][distance='1']/frr-nexthops/nexthop[nh-type='blackhole'][vrf='default'][gateway=''][interface='(null)']/vrf default",
                "mgmt commit abort",
            ]
        }
    }

    result = apply_raw_config(tgen, raw_config)
    assert result is True, "Testcase {} : Failed Error: {}".format(tc_name, result)

    step("verify that the route is not configured")

    dut = "r1"
    protocol = "static"
    input_dict_4 = {
        "r2": {
            "static_routes": [
                {
                    "network": "192.1.1.3/32",
                    "next_hop": "Null0",
                }
            ]
        }
    }
    result = verify_rib(
        tgen, "ipv4", dut, input_dict_4, protocol=protocol, expected=False
    )
    assert (
        result is not True
    ), "Testcase {} : Failed" "Error: Routes is missing in RIB".format(tc_name)

    write_test_footer(tc_name)


def test_mgmt_delete_config(request):
    """
    Verify mgmt delete config.

    """
    tc_name = request.node.name
    write_test_header(tc_name)
    tgen = get_topogen()
    # Don't run this test if we have any failure.
    if tgen.routers_have_failure():
        pytest.skip(tgen.errors)

    reset_config_on_routers(tgen)

    step("Mgmt - Configure a static route using commit apply")

    raw_config = {
        "r1": {
            "raw_config": [
                "mgmt set-config /frr-routing:routing/control-plane-protocols/control-plane-protocol[type='frr-staticd:staticd'][name='staticd'][vrf='default']/frr-staticd:staticd/route-list[prefix='192.168.1.3/32'][src-prefix='::/0'][afi-safi='frr-routing:ipv4-unicast']/path-list[table-id='0'][distance='1']/frr-nexthops/nexthop[nh-type='blackhole'][vrf='default'][gateway=''][interface='(null)']/vrf default",
                "mgmt commit apply",
            ]
        }
    }

    result = apply_raw_config(tgen, raw_config)
    assert result is True, "Testcase {} : Failed Error: {}".format(tc_name, result)

    step("Verify that the route is added to RIB")
    dut = "r1"
    protocol = "static"
    input_dict_4 = {
        "r2": {
            "static_routes": [
                {
                    "network": "192.168.1.3/32",
                    "next_hop": "Null0",
                }
            ]
        }
    }
    result = verify_rib(tgen, "ipv4", dut, input_dict_4, protocol=protocol)
    assert (
        result is True
    ), "Testcase {} : Failed" "Error: Routes is missing in RIB".format(tc_name)

    step("Mgmt delete config")
    raw_config = {
        "r1": {
            "raw_config": [
                "mgmt delete-config /frr-routing:routing/control-plane-protocols/control-plane-protocol[type='frr-staticd:staticd'][name='staticd'][vrf='default']/frr-staticd:staticd/route-list[prefix='192.168.1.3/32'][src-prefix='::/0'][afi-safi='frr-routing:ipv4-unicast']",
                "mgmt commit apply",
            ]
        }
    }

    result = apply_raw_config(tgen, raw_config)
    assert result is True, "Testcase {} : Failed Error: {}".format(tc_name, result)

    step("Verify that the route is deleted from RIB")
    result = verify_rib(
        tgen, "ipv4", dut, input_dict_4, protocol=protocol, expected=False
    )
    assert (
        result is not True
    ), "Testcase {} : Failed" "Error: Routes is still present in RIB".format(tc_name)

    write_test_footer(tc_name)


def test_mgmt_edit_config(request):
    """
    Verify mgmt edit config.
    """
    tc_name = request.node.name
    write_test_header(tc_name)
    tgen = get_topogen()
    # Don't run this test if we have any failure.
    if tgen.routers_have_failure():
        pytest.skip(tgen.errors)

    reset_config_on_routers(tgen)

    r1 = tgen.gears["r1"]

    # check "create" operation
    data = {"frr-interface:interface": [{"name": "eth0", "description": "eth0-desc"}]}
    r1.vtysh_cmd(
        f"conf\nmgmt edit create /frr-interface:lib lock commit {json.dumps(data, separators=(',', ':'))}"
    )
    data_out = data
    assert (
        router_json_cmp(
            r1,
            "show mgmt get-data /frr-interface:lib/interface[name='eth0'] only-config exact",
            data_out,
            exact=True,
        )
        == None
    )

    # check error on "create" for an existing object
    data = {"frr-interface:interface": [{"name": "eth0", "description": "eth0-desc"}]}
    ret = r1.vtysh_cmd(
        f"conf\nmgmt edit create /frr-interface:lib lock commit {json.dumps(data, separators=(',', ':'))}"
    )
    assert "Data already exists" in ret

    # check adding a leaf to an existing object using "merge"
    data = {
        "frr-interface:interface": [
            {"name": "eth0", "frr-zebra:zebra": {"bandwidth": 100}}
        ]
    }
    r1.vtysh_cmd(
        f"conf\nmgmt edit merge /frr-interface:lib/interface[name='eth0'] lock commit {json.dumps(data, separators=(',', ':'))}"
    )
    data_out = {
        "frr-interface:interface": [
            {
                "name": "eth0",
                "description": "eth0-desc",
                "frr-zebra:zebra": {"bandwidth": 100},
            }
        ]
    }
    assert (
        router_json_cmp(
            r1,
            "show mgmt get-data /frr-interface:lib/interface[name='eth0'] only-config exact",
            data_out,
            exact=True,
        )
        == None
    )

    # check replacing an existing object using "replace"
    data = {
        "frr-interface:interface": [{"name": "eth0", "description": "eth0-desc-new"}]
    }
    r1.vtysh_cmd(
        f"conf\nmgmt edit replace /frr-interface:lib/interface[name='eth0'] lock commit {json.dumps(data, separators=(',', ':'))}"
    )
    data_out = data
    assert (
        router_json_cmp(
            r1,
            "show mgmt get-data /frr-interface:lib/interface[name='eth0'] only-config exact",
            data_out,
            exact=True,
        )
        == None
    )

    # check error on "replace" when keys in xpath and data are different
    data = {
        "frr-interface:interface": [{"name": "eth1", "description": "eth0-desc-new"}]
    }
    ret = r1.vtysh_cmd(
        f"conf\nmgmt edit replace /frr-interface:lib/interface[name='eth0'] lock commit {json.dumps(data, separators=(',', ':'))}"
    )
    assert "List keys in xpath and data tree are different" in ret

    # check deleting an existing object using "delete"
    r1.vtysh_cmd(
        f"conf\nmgmt edit delete /frr-interface:lib/interface[name='eth0'] lock commit"
    )
    assert (
        router_json_cmp(
            r1,
            "show mgmt get-data /frr-interface:lib/interface[name='eth0'] only-config exact",
            {},
            exact=True,
        )
        == None
    )

    # check error on "delete" for a non-existing object
    ret = r1.vtysh_cmd(
        f"conf\nmgmt edit delete /frr-interface:lib/interface[name='eth0'] lock commit"
    )
    assert "Data missing" in ret

    # check no error on "remove" for a non-existing object
    ret = r1.vtysh_cmd(
        f"conf\nmgmt edit remove /frr-interface:lib/interface[name='eth0'] lock commit"
    )
    assert "Data missing" not in ret

    # check "remove" for an existing object
    data = {"frr-interface:interface": [{"name": "eth0", "description": "eth0-desc"}]}
    r1.vtysh_cmd(
        f"conf\nmgmt edit create /frr-interface:lib lock commit {json.dumps(data, separators=(',', ':'))}"
    )
    r1.vtysh_cmd(
        f"conf\nmgmt edit remove /frr-interface:lib/interface[name='eth0'] lock commit"
    )
    assert (
        router_json_cmp(
            r1,
            "show mgmt get-data /frr-interface:lib/interface[name='eth0'] only-config exact",
            {},
            exact=True,
        )
        == None
    )

    # check "create" of a top-level node
    data = {"frr-vrf:lib": {"vrf": [{"name": "vrf1"}]}}
    r1.vtysh_cmd(
        f"conf\nmgmt edit create / lock commit {json.dumps(data, separators=(',', ':'))}"
    )
    data_out = data
    assert (
        router_json_cmp(
            r1,
            "show mgmt get-data /frr-vrf:lib only-config exact",
            data_out,
            exact=True,
        )
        == None
    )

    # check "replace" of a top-level node
    data = {"frr-vrf:lib": {"vrf": [{"name": "vrf2"}]}}
    r1.vtysh_cmd(
        f"conf\nmgmt edit replace /frr-vrf:lib lock commit {json.dumps(data, separators=(',', ':'))}"
    )
    data_out = data
    assert (
        router_json_cmp(
            r1,
            "show mgmt get-data /frr-vrf:lib only-config exact",
            data_out,
            exact=True,
        )
        == None
    )

    # check "delete" of a top-level node
    r1.vtysh_cmd(f"conf\nmgmt edit delete /frr-vrf:lib lock commit")
    assert (
        router_json_cmp(
            r1, "show mgmt get-data /frr-vrf:lib only-config exact", {}, exact=True
        )
        == None
    )

    # check replace of the whole config
    #   this test won't work at the moment, because we don't allow to delete
    #   interfaces from the config if they are present in the system
    #   another problem is that we don't allow "/" as an xpath in get-data command
    #   therefore, commenting it out for now
    # data = {
    #     "frr-interface:lib": {
    #         "interface": [{"name": "eth1", "description": "eth1-desc"}]
    #     },
    #     "frr-vrf:lib": {"vrf": [{"name": "vrf3"}]},
    # }
    # r1.vtysh_cmd(
    #     f"conf\nmgmt edit replace / lock commit {json.dumps(data, separators=(',', ':'))}"
    # )
    # data_out = data
    # assert (
    #     router_json_cmp(
    #         r1,
    #         "show mgmt get-data / only-config exact",
    #         data_out,
    #         exact=True,
    #     )
    #     == None
    # )

    # check "merge" of the whole config
    data = {
        "frr-interface:lib": {
            "interface": [{"name": "eth2", "description": "eth2-desc"}]
        },
        "frr-vrf:lib": {"vrf": [{"name": "vrf4"}]},
    }
    r1.vtysh_cmd(
        f"conf\nmgmt edit merge / lock commit {json.dumps(data, separators=(',', ':'))}"
    )
    data_out = data
    assert (
        router_json_cmp(
            r1,
            "show mgmt get-data /frr-interface:lib only-config exact",
            {
                "frr-interface:lib": {
                    "interface": [{"name": "eth2", "description": "eth2-desc"}]
                }
            },
        )
        == None
    )
    assert (
        router_json_cmp(
            r1,
            "show mgmt get-data /frr-vrf:lib only-config exact",
            {"frr-vrf:lib": {"vrf": [{"name": "vrf4"}]}},
        )
        == None
    )


def test_mgmt_chaos_stop_start_frr(request):
    """
    Kill mgmtd - verify that watch frr restarts.

    """
    tc_name = request.node.name
    write_test_header(tc_name)
    tgen = get_topogen()
    # Don't run this test if we have any failure.
    if tgen.routers_have_failure():
        pytest.skip(tgen.errors)

    reset_config_on_routers(tgen)
    next_hop_ip = populate_nh()

    step("Configure Static route with next hop null 0")

    raw_config = {
        "r1": {
            "raw_config": [
                "mgmt set-config /frr-routing:routing/control-plane-protocols/control-plane-protocol[type='frr-staticd:staticd'][name='staticd'][vrf='default']/frr-staticd:staticd/route-list[prefix='192.1.11.200/32'][src-prefix='::/0'][afi-safi='frr-routing:ipv4-unicast']/path-list[table-id='0'][distance='1']/frr-nexthops/nexthop[nh-type='blackhole'][vrf='default'][gateway=''][interface='(null)']/bh-type unspec",
                "mgmt commit apply",
            ]
        }
    }

    result = apply_raw_config(tgen, raw_config)
    assert result is True, "Testcase {} : Failed Error: {}".format(tc_name, result)

    step("verify that the route is configured and present in the zebra")

    dut = "r1"
    protocol = "static"
    input_dict_4 = {"r2": {"static_routes": [{"network": "192.1.11.200/32"}]}}
    result = verify_rib(tgen, "ipv4", dut, input_dict_4, protocol=protocol)
    assert (
        result is True
    ), "Testcase {} : Failed" "Error: Routes is missing in RIB".format(tc_name)

    step("Restart frr")
    stop_router(tgen, "r1")
    start_router(tgen, "r1")
    step("Verify routes are intact in zebra.")
    result = verify_rib(tgen, "ipv4", dut, input_dict_4, protocol=protocol)
    assert (
        result is True
    ), "Testcase {} : Failed" "Error: Routes is missing in RIB".format(tc_name)

    step("delete the configured route and ")
    raw_config = {
        "r1": {
            "raw_config": [
                "mgmt  delete-config /frr-routing:routing/control-plane-protocols/control-plane-protocol[type='frr-staticd:staticd'][name='staticd'][vrf='default']/frr-staticd:staticd/route-list[prefix='192.1.11.200/32'][src-prefix='::/0'][afi-safi='frr-routing:ipv4-unicast']",
                "mgmt commit apply",
            ]
        }
    }

    result = apply_raw_config(tgen, raw_config)
    assert result is True, "Testcase {} : Failed Error: {}".format(tc_name, result)

    step("verify that the route is deleted and deleted from zebra")

    dut = "r1"
    protocol = "static"
    input_dict_4 = {"r1": {"static_routes": [{"network": "192.1.11.200/32"}]}}
    result = verify_rib(
        tgen, "ipv4", dut, input_dict_4, protocol=protocol, expected=False
    )
    assert (
        result is not True
    ), "Testcase {} : Failed" "Error: Routes still present in RIB".format(tc_name)

    write_test_footer(tc_name)


def test_mgmt_chaos_kill_daemon(request):
    """
    Kill mgmtd - verify that static routes are intact

    """
    tc_name = request.node.name
    write_test_header(tc_name)
    tgen = get_topogen()
    # Don't run this test if we have any failure.
    if tgen.routers_have_failure():
        pytest.skip(tgen.errors)

    reset_config_on_routers(tgen)
    next_hop_ip = populate_nh()

    step("Configure Static route with next hop null 0")

    raw_config = {
        "r1": {
            "raw_config": [
                "mgmt set-config /frr-routing:routing/control-plane-protocols/control-plane-protocol[type='frr-staticd:staticd'][name='staticd'][vrf='default']/frr-staticd:staticd/route-list[prefix='192.1.11.200/32'][src-prefix='::/0'][afi-safi='frr-routing:ipv4-unicast']/path-list[table-id='0'][distance='1']/frr-nexthops/nexthop[nh-type='blackhole'][vrf='default'][gateway=''][interface='(null)']/bh-type unspec",
                "mgmt commit apply",
            ]
        }
    }

    result = apply_raw_config(tgen, raw_config)
    assert result is True, "Testcase {} : Failed Error: {}".format(tc_name, result)

    step("verify that the route is configured and present in the zebra")

    dut = "r1"
    protocol = "static"
    input_dict_4 = {"r2": {"static_routes": [{"network": "192.1.11.200/32"}]}}
    result = verify_rib(tgen, "ipv4", dut, input_dict_4, protocol=protocol)
    assert (
        result is True
    ), "Testcase {} : Failed" "Error: Routes is missing in RIB".format(tc_name)

    step("Kill static daemon on R2.")
    kill_router_daemons(tgen, "r1", ["staticd"])

    step("Bring up staticd daemon on R2.")
    start_router_daemons(tgen, "r1", ["staticd"])

    step("Verify routes are intact in zebra.")
    result = verify_rib(tgen, "ipv4", dut, input_dict_4, protocol=protocol)
    assert (
        result is True
    ), "Testcase {} : Failed" "Error: Routes is missing in RIB".format(tc_name)

    step("Kill mgmt daemon on R2.")
    kill_router_daemons(tgen, "r1", ["mgmtd"])

    step("Bring up zebra daemon on R2.")
    start_router_daemons(tgen, "r1", ["mgmtd"])

    step("Verify routes are intact in zebra.")
    result = verify_rib(tgen, "ipv4", dut, input_dict_4, protocol=protocol)
    assert (
        result is True
    ), "Testcase {} : Failed" "Error: Routes is missing in RIB".format(tc_name)

    write_test_footer(tc_name)


if __name__ == "__main__":
    args = ["-s"] + sys.argv[1:]
    sys.exit(pytest.main(args))



from lib.lutil import luCommand

luCommand(
    "ce1",
    'vtysh -c "show bgp ipv4 uni"',
    "7 routes and 7",
    "wait",
    "Local and remote routes",
)
luCommand(
    "ce2",
    'vtysh -c "show bgp ipv4 uni"',
    "7 routes and 9",
    "wait",
    "Local and remote routes",
)
luCommand(
    "ce3",
    'vtysh -c "show bgp ipv4 uni"',
    "7 routes and 7",
    "wait",
    "Local and remote routes",
)
luCommand(
    "r1", 'vtysh -c "show bgp ipv4 uni"', "7 routes and 9", "pass", "Unicast SAFI"
)
luCommand(
    "r2",
    'vtysh -c "show bgp ipv4 uni"',
    "No BGP prefixes displayed",
    "pass",
    "Unicast SAFI",
)
luCommand(
    "r3", 'vtysh -c "show bgp ipv4 uni"', "7 routes and 9", "pass", "Unicast SAFI"
)
luCommand(
    "r4", 'vtysh -c "show bgp ipv4 uni"', "7 routes and 9", "pass", "Unicast SAFI"
)
have2ndImports = luCommand(
    "r3",
    'vtysh -c "show vnc registrations imported"',
    "2 out of 2 imported",
    "none",
    "Imported Registrations",
    2,
)
if have2ndImports:
    num = "9 routes and 9"
else:
    num = "7 routes and 7"
luCommand("r1", 'vtysh -c "show bgp ipv4 vpn"', num, "pass", "VPN SAFI")
luCommand("r2", 'vtysh -c "show bgp ipv4 vpn"', num, "pass", "VPN SAFI")
luCommand("r3", 'vtysh -c "show bgp ipv4 vpn"', num, "pass", "VPN SAFI")
luCommand("r4", 'vtysh -c "show bgp ipv4 vpn"', num, "pass", "VPN SAFI")



#!/usr/bin/env python
# SPDX-License-Identifier: ISC

#
# test_run_and_expect.py
# Tests for library function: run_and_expect(_type)().
#
# Copyright (c) 2019 by
# Network Device Education Foundation, Inc. ("NetDEF")
#

"""
Tests for the `run_and_expect(_type)()` functions.
"""

import os
import sys
import pytest

# Save the Current Working Directory to find lib files.
CWD = os.path.dirname(os.path.realpath(__file__))
sys.path.append(os.path.join(CWD, "../../"))

# pylint: disable=C0413
from lib.topotest import run_and_expect_type


def test_run_and_expect_type():
    "Test basic `run_and_expect_type` functionality."

    def return_true():
        "Test function that returns `True`."
        return True

    # Test value success.
    success, value = run_and_expect_type(
        return_true, bool, count=1, wait=0, avalue=True
    )
    assert success is True
    assert value is True

    # Test value failure.
    success, value = run_and_expect_type(
        return_true, bool, count=1, wait=0, avalue=False
    )
    assert success is False
    assert value is True

    # Test type success.
    success, value = run_and_expect_type(return_true, bool, count=1, wait=0)
    assert success is True
    assert value is True

    # Test type failure.
    success, value = run_and_expect_type(return_true, str, count=1, wait=0)
    assert success is False
    assert value is True

    # Test type failure, return correct type.
    success, value = run_and_expect_type(return_true, str, count=1, wait=0, avalue=True)
    assert success is False
    assert value is True


if __name__ == "__main__":
    sys.exit(pytest.main())



#!/usr/bin/env python
# SPDX-License-Identifier: GPL-2.0-or-later
#
# April 03 2023, Trey Aspelund <taspelund@nvidia.com>
#
# Copyright (C) 2023 NVIDIA Corporation
#
# Test if the CLI parser for RT/SoO ecoms correctly
# constrain user input to valid 4-byte ASN values.
#

import os
import sys
import pytest
import functools

CWD = os.path.dirname(os.path.realpath(__file__))
sys.path.append(os.path.join(CWD, "../"))

# pylint: disable=C0413
from lib import topotest
from lib.topogen import Topogen, TopoRouter, get_topogen
from lib.common_config import step

pytestmark = [pytest.mark.bgpd]


def build_topo(tgen):
    tgen.add_router("pe1")


def setup_module(mod):
    tgen = Topogen(build_topo, mod.__name__)
    tgen.start_topology()
    pe1 = tgen.gears["pe1"]
    pe1.load_config(TopoRouter.RD_BGP, os.path.join(CWD, "pe1/bgpd.conf"))
    tgen.start_router()


def teardown_module(mod):
    tgen = get_topogen()
    tgen.stop_topology()


def test_bgp_route_origin_parser():
    tgen = get_topogen()

    if tgen.routers_have_failure():
        pytest.skip(tgen.errors)

    pe1 = tgen.gears["pe1"]

    def _invalid_soo_accepted():
        pe1.vtysh_cmd(
            """
        configure terminal
        router bgp 65001
         address-family ipv4 unicast
          neighbor 192.168.2.1 soo 4294967296:65
        """
        )
        run_cfg = pe1.vtysh_cmd("show run")
        return "soo" in run_cfg

    def _max_soo_accepted():
        pe1.vtysh_cmd(
            """
        configure terminal
        router bgp 65001
         address-family ipv4 unicast
          neighbor 192.168.2.1 soo 4294967295:65
            """
        )
        run_cfg = pe1.vtysh_cmd("show run")
        return "soo 4294967295:65" in run_cfg

    def _invalid_rt_accepted():
        pe1.vtysh_cmd(
            """
        configure terminal
        router bgp 65001
         address-family ipv4 unicast
          rt vpn both 4294967296:65
        """
        )
        run_cfg = pe1.vtysh_cmd("show run")
        return "rt vpn" in run_cfg

    def _max_rt_accepted():
        pe1.vtysh_cmd(
            """
        configure terminal
        router bgp 65001
         address-family ipv4 unicast
          rt vpn both 4294967295:65
            """
        )
        run_cfg = pe1.vtysh_cmd("show run")
        return "rt vpn both 4294967295:65" in run_cfg

    step(
        "Configure invalid 4-byte value SoO (4294967296:65), this should not be accepted"
    )
    test_func = functools.partial(_invalid_soo_accepted)
    _, result = topotest.run_and_expect(test_func, False, count=30, wait=0.5)
    assert result is False, "invalid 4-byte value of SoO accepted"

    step("Configure max 4-byte value SoO (4294967295:65), this should be accepted")
    test_func = functools.partial(_max_soo_accepted)
    _, result = topotest.run_and_expect(test_func, True, count=30, wait=0.5)
    assert result is True, "max 4-byte value of SoO not accepted"

    step(
        "Configure invalid 4-byte value RT (4294967296:65), this should not be accepted"
    )
    test_func = functools.partial(_invalid_rt_accepted)
    _, result = topotest.run_and_expect(test_func, False, count=30, wait=0.5)
    assert result is False, "invalid 4-byte value of RT accepted"

    step("Configure max 4-byte value RT (4294967295:65), this should be accepted")
    test_func = functools.partial(_max_rt_accepted)
    _, result = topotest.run_and_expect(test_func, True, count=30, wait=0.5)
    assert result is True, "max 4-byte value of RT not accepted"


if __name__ == "__main__":
    args = ["-s"] + sys.argv[1:]
    sys.exit(pytest.main(args))



#!/usr/bin/env python
# SPDX-License-Identifier: ISC

#
# test_zebra_multiple_connected.py
#
# Copyright (c) 2022 by
# Nvidia Corporation
# Donald Sharp
#

"""
test_zebra_multiple_connected.py: Testing multiple connected

"""

import os
import sys
import pytest
import json
from functools import partial
from lib.topolog import logger

pytestmark = pytest.mark.random_order(disabled=True)

# Save the Current Working Directory to find configuration files.
CWD = os.path.dirname(os.path.realpath(__file__))
sys.path.append(os.path.join(CWD, "../"))

# pylint: disable=C0413
# Import topogen and topotest helpers
from lib import topotest
from lib.topogen import Topogen, TopoRouter, get_topogen

# Required to instantiate the topology builder class.

#####################################################
##
##   Network Topology Definition
##
#####################################################


def build_topo(tgen):
    for routern in range(1, 4):
        tgen.add_router("r{}".format(routern))

    # On main router
    # First switch is for a dummy interface (for local network)
    switch = tgen.add_switch("sw1")
    switch.add_link(tgen.gears["r1"])

    # Switches for zebra
    # switch 2 switch is for connection to zebra router
    switch = tgen.add_switch("sw2")
    switch.add_link(tgen.gears["r1"])
    switch.add_link(tgen.gears["r2"])

    # switch 4 is stub on remote zebra router
    switch = tgen.add_switch("sw4")
    switch.add_link(tgen.gears["r3"])

    # switch 3 is between zebra routers
    switch = tgen.add_switch("sw3")
    switch.add_link(tgen.gears["r2"])
    switch.add_link(tgen.gears["r3"])

    # Create a p2p connection between r1 and r2
    tgen.add_link(tgen.gears["r1"], tgen.gears["r2"])


#####################################################
##
##   Tests starting
##
#####################################################


def setup_module(module):
    "Setup topology"
    tgen = Topogen(build_topo, module.__name__)
    tgen.start_topology()

    # This is a sample of configuration loading.
    router_list = tgen.routers()
    for rname, router in router_list.items():
        router.load_config(
            TopoRouter.RD_ZEBRA, os.path.join(CWD, "{}/zebra.conf".format(rname))
        )

    tgen.start_router()


def teardown_module(_mod):
    "Teardown the pytest environment"
    tgen = get_topogen()

    # This function tears down the whole topology.
    tgen.stop_topology()


def test_zebra_connected_multiple():
    "Test multiple connected routes that have a kernel route pointing at one"

    tgen = get_topogen()
    # Don't run this test if we have any failure.
    if tgen.routers_have_failure():
        pytest.skip(tgen.errors)

    router = tgen.gears["r1"]
    router.run("ip route add 192.168.1.1/32 via 10.0.1.99 dev r1-eth1")
    router.run("ip link add dummy1 type dummy")
    router.run("ip link set dummy1 up")
    router.run("ip link set dummy1 down")

    routes = "{}/{}/ip_route.json".format(CWD, router.name)
    expected = json.loads(open(routes).read())

    test_func = partial(
        topotest.router_json_cmp, router, "show ip route json", expected
    )

    _, result = topotest.run_and_expect(test_func, None, count=20, wait=1)
    assert result is None, "Kernel route is missing from zebra"


def test_zebra_system_recursion():
    "Test a system route recursing through another system route"

    tgen = get_topogen()
    if tgen.routers_have_failure():
        pytest.skip(tgen.errors)

    router = tgen.gears["r1"]
    router.run("ip route add 10.0.1.30/32 dev r1-eth1")
    router.run("ip route add 10.9.9.0/24 via 10.0.1.30 dev r1-eth1")
    router.run("ip link add dummy2 type dummy")
    router.run("ip link set dummy2 up")
    router.run("ip link set dummy2 down")

    routes = "{}/{}/ip_route2.json".format(CWD, router.name)
    expected = json.loads(open(routes).read())
    test_func = partial(
        topotest.router_json_cmp, router, "show ip route json", expected
    )

    _, result = topotest.run_and_expect(test_func, None, count=20, wait=1)
    assert result is None, "Kernel route is missing from zebra"


def test_zebra_noprefix_connected():
    "Test that a noprefixroute created does not create a connected route"

    tgen = get_topogen()
    if tgen.routers_have_failure():
        pytest.skip(tgen.errors)

    router = tgen.gears["r1"]
    router.run("ip addr add 192.168.44.1/24 dev r1-eth1 noprefixroute")
    expected = "% Network not in table"
    test_func = partial(
        topotest.router_output_cmp, router, "show ip route 192.168.44.0/24", expected
    )
    result, _ = topotest.run_and_expect(test_func, "", count=20, wait=1)
    assert result, "Connected Route should not have been added"


def test_zebra_noprefix_connected_add():
    "Test that a noprefixroute created with a manual route works as expected, this is for NetworkManager"

    tgen = get_topogen()
    if tgen.routers_have_failure():
        pytest.skip(tgen.errors)

    router = tgen.gears["r1"]
    router.run("ip route add 192.168.44.0/24 dev r1-eth1")

    connected = "{}/{}/ip_route_connected.json".format(CWD, router.name)
    expected = json.loads(open(connected).read())

    test_func = partial(
        topotest.router_json_cmp, router, "show ip route 192.168.44.0/24 json", expected
    )
    result, _ = topotest.run_and_expect(test_func, None, count=20, wait=1)
    assert result, "Connected Route should have been added\n{}".format(_)


def test_zebra_kernel_route_add():
    "Test that a random kernel route is properly handled as expected"

    tgen = get_topogen()
    if tgen.routers_have_failure():
        pytest.skip(tgen.errors)

    router = tgen.gears["r1"]
    router.run("ip route add 4.5.6.7/32 dev r1-eth1")

    kernel = "{}/{}/ip_route_kernel.json".format(CWD, router.name)
    expected = json.loads(open(kernel).read())

    test_func = partial(
        topotest.router_json_cmp, router, "show ip route 4.5.6.7/32 json", expected
    )
    result, _ = topotest.run_and_expect(test_func, None, count=20, wait=1)
    assert result, "Connected Route should have been added\n{}".format(_)


def test_zebra_kernel_route_blackhole_add():
    "Test that a blackhole route is not affected by interface's link change"

    tgen = get_topogen()
    if tgen.routers_have_failure():
        pytest.skip(tgen.errors)

    router = tgen.gears["r1"]
    router.run("ip route add blackhole default")
    router.run("ip link set dev r1-eth1 down")

    kernel = "{}/{}/ip_route_kernel_blackhole.json".format(CWD, router.name)
    expected = json.loads(open(kernel).read())

    test_func = partial(
        topotest.router_json_cmp, router, "show ip route 0.0.0.0/0 json", expected
    )
    result, _ = topotest.run_and_expect(test_func, None, count=20, wait=1)
    assert result, "Blackhole Route should have not been removed\n{}".format(_)

def test_zebra_kernel_route_interface_linkdown():
    "Test that a kernel routes should be affected by interface change"

    tgen = get_topogen()
    if tgen.routers_have_failure():
        pytest.skip(tgen.errors)

    router = tgen.gears["r1"]
    router.run("ip route add 5.5.6.7/32 via 10.0.1.66 dev r1-eth2")

    kernel = "{}/{}/ip_route_kernel_interface_up.json".format(CWD, router.name)
    expected = json.loads(open(kernel).read())

    test_func = partial(
            topotest.router_json_cmp, router, "show ip route 5.5.6.7/32 json", expected
            )
    result, _ = topotest.run_and_expect(test_func, None, count=20, wait=1)
    assert result, "Kernel Route should be selected:\n{}".format(_)

    # link down
    router2 = tgen.gears["r2"]
    router2.run("ip link set dev r2-eth2 down")

    kernel = "{}/{}/ip_route_kernel_interface_down.json".format(CWD, router.name)
    expected = json.loads(open(kernel).read())

    test_func = partial(
        topotest.router_json_cmp, router, "show ip route 5.5.6.7/32 json", expected
    )
    result, _ = topotest.run_and_expect(test_func, None, count=20, wait=1)
    assert result, "Kernel Route should not be selected:\n{}".format(_)

    # link up
    router2 = tgen.gears["r2"]
    router2.run("ip link set dev r2-eth2 up")

    kernel = "{}/{}/ip_route_kernel_interface_up.json".format(CWD, router.name)
    expected = json.loads(open(kernel).read())

    test_func = partial(
            topotest.router_json_cmp, router, "show ip route 5.5.6.7/32 json", expected
            )
    result, _ = topotest.run_and_expect(test_func, None, count=20, wait=1)
    assert result, "Kernel Route should be selected:\n{}".format(_)

if __name__ == "__main__":
    args = ["-s"] + sys.argv[1:]
    sys.exit(pytest.main(args))



#!/usr/bin/env python
# SPDX-License-Identifier: ISC

# Copyright (c) 2022 by
# Donatas Abraitis <donatas@opensourcerouting.org>
#

"""
Test if Software Version capability works if forced with a knob.
Reference: https://datatracker.ietf.org/doc/html/draft-abraitis-bgp-version-capability
"""

import os
import re
import sys
import json
import pytest
import functools

pytestmark = [pytest.mark.bgpd]

CWD = os.path.dirname(os.path.realpath(__file__))
sys.path.append(os.path.join(CWD, "../"))

# pylint: disable=C0413
from lib import topotest
from lib.topogen import Topogen, TopoRouter, get_topogen


def setup_module(mod):
    topodef = {"s1": ("r1", "r2")}
    tgen = Topogen(topodef, mod.__name__)
    tgen.start_topology()

    router_list = tgen.routers()

    for _, (rname, router) in enumerate(router_list.items(), 1):
        router.load_config(
            TopoRouter.RD_ZEBRA, os.path.join(CWD, "{}/zebra.conf".format(rname))
        )
        router.load_config(
            TopoRouter.RD_BGP, os.path.join(CWD, "{}/bgpd.conf".format(rname))
        )

    tgen.start_router()


def teardown_module(mod):
    tgen = get_topogen()
    tgen.stop_topology()


def test_bgp_software_version():
    tgen = get_topogen()

    if tgen.routers_have_failure():
        pytest.skip(tgen.errors)

    r1 = tgen.gears["r1"]

    def _bgp_converge():
        output = json.loads(r1.vtysh_cmd("show bgp summary json"))
        expected = {"ipv4Unicast": {"peers": {"192.168.1.2": {"state": "Established"}}}}
        return topotest.json_cmp(output, expected)

    test_func = functools.partial(
        _bgp_converge,
    )
    _, result = topotest.run_and_expect(test_func, None, count=60, wait=0.5)
    assert result is None, "Can't converge"

    def _bgp_check_software_version():
        output = json.loads(r1.vtysh_cmd("show bgp neighbor 192.168.1.2 json"))

        try:
            versions = output["192.168.1.2"]["neighborCapabilities"]["softwareVersion"]
            adv = versions["advertisedSoftwareVersion"]
            rcv = versions["receivedSoftwareVersion"]

            if not adv and not rcv:
                return False

            pattern = "^FRRouting/\\d.+"
            if re.search(pattern, adv) and re.search(pattern, rcv):
                return True
        except:
            return False

        return False

    assert _bgp_check_software_version(), "Neighbor's software version is n/a"


if __name__ == "__main__":
    args = ["-s"] + sys.argv[1:]
    sys.exit(pytest.main(args))



#!/usr/bin/env python
# SPDX-License-Identifier: ISC

# Copyright (c) 2023 by
# Donatas Abraitis <donatas@opensourcerouting.org>
#

"""
Test if static route with BGP conditional advertisement works correctly
if we modify the prefix-lists.
"""

import os
import sys
import json
import pytest
import functools

pytestmark = [pytest.mark.bgpd]

CWD = os.path.dirname(os.path.realpath(__file__))
sys.path.append(os.path.join(CWD, "../"))

# pylint: disable=C0413
from lib import topotest
from lib.topogen import Topogen, get_topogen
from lib.common_config import step


def setup_module(mod):
    topodef = {"s1": ("r1", "r2"), "s2": ("r2", "r3")}
    tgen = Topogen(topodef, mod.__name__)
    tgen.start_topology()

    router_list = tgen.routers()

    for _, (rname, router) in enumerate(router_list.items(), 1):
        router.load_frr_config(os.path.join(CWD, "{}/frr.conf".format(rname)))

    tgen.start_router()


def teardown_module(mod):
    tgen = get_topogen()
    tgen.stop_topology()


def test_bgp_conditional_advertisements_static_route():
    tgen = get_topogen()

    if tgen.routers_have_failure():
        pytest.skip(tgen.errors)

    r1 = tgen.gears["r1"]
    r2 = tgen.gears["r2"]

    def _bgp_converge():
        output = json.loads(
            r2.vtysh_cmd(
                "show bgp ipv4 unicast neighbor 192.168.1.1 advertised-routes json"
            )
        )
        expected = {
            "advertisedRoutes": {
                "10.10.10.1/32": {
                    "valid": True,
                }
            },
            "totalPrefixCounter": 1,
        }
        return topotest.json_cmp(output, expected)

    test_func = functools.partial(
        _bgp_converge,
    )
    _, result = topotest.run_and_expect(test_func, None, count=30, wait=1)
    assert result is None, "Can't converge"

    step("Append prefix-list to advertise 10.10.10.2/32")

    r2.vtysh_cmd(
        """
    configure terminal
        ip prefix-list advertise seq 10 permit 10.10.10.2/32
    """
    )

    def _bgp_check_advertised_after_update():
        output = json.loads(
            r2.vtysh_cmd(
                "show bgp ipv4 unicast neighbor 192.168.1.1 advertised-routes json"
            )
        )
        expected = {
            "advertisedRoutes": {
                "10.10.10.1/32": {
                    "valid": True,
                },
                "10.10.10.2/32": {
                    "valid": True,
                },
            },
            "totalPrefixCounter": 2,
        }
        return topotest.json_cmp(output, expected)

    test_func = functools.partial(
        _bgp_check_advertised_after_update,
    )
    _, result = topotest.run_and_expect(test_func, None, count=30, wait=1)
    assert result is None, "10.10.10.2/32 is not advertised after prefix-list update"

    def _bgp_check_received_routes():
        output = json.loads(r1.vtysh_cmd("show bgp ipv4 unicast 10.10.10.1/32 json"))
        expected = {
            "paths": [
                {
                    "community": {
                        "string": "65000:1",
                    }
                }
            ]
        }
        return topotest.json_cmp(output, expected)

    test_func = functools.partial(
        _bgp_check_received_routes,
    )
    _, result = topotest.run_and_expect(test_func, None, count=30, wait=1)
    assert result is None, "10.10.10.1/32 does not have 65000:1 community attached"


if __name__ == "__main__":
    args = ["-s"] + sys.argv[1:]
    sys.exit(pytest.main(args))



#!/usr/bin/env python
# SPDX-License-Identifier: ISC

#
# Copyright (c) 2022 by VMware, Inc. ("VMware")
# Used Copyright (c) 2018 by Network Device Education Foundation,
# Inc. ("NetDEF") in this file.
#

import sys
import time
import pytest
import inspect
import os
from copy import deepcopy

# Save the Current Working Directory to find configuration files.
CWD = os.path.dirname(os.path.realpath(__file__))
sys.path.append(os.path.join(CWD, "../"))
sys.path.append(os.path.join(CWD, "../lib/"))

"""Following tests are covered to test bgp unique rid functionality.
1. Verify iBGP session when same and different router ID is configured in user VRF(GREEN).
2. Verify eBGP session when same and different router ID is configured in user vrf (VRF RED)
3. Verify two different eBGP sessions initiated with same router ID in user VRf (RED and GREEN)
"""

#################################
# TOPOLOGY
#################################
"""

                    +-------+
         +--------- |  R2   |
         |          +-------+
         |iBGP           |
     +-------+           |
     |  R1   |           |iBGP
     +-------+           |
         |               |
         |    iBGP   +-------+   eBGP   +-------+
         +---------- |  R3   |========= |  R4   |
                     +-------+          +-------+
                        |
                        |eBGP
                        |
                    +-------+
                    |  R5   |
                    +-------+


"""

# pylint: disable=C0413
# Import topogen and topotest helpers
from lib.topogen import Topogen, get_topogen
from lib.topojson import build_config_from_json
from lib.topolog import logger

pytestmark = [pytest.mark.bgpd, pytest.mark.staticd]

# Required to instantiate the topology builder class.
from lib.common_config import (
    start_topology,
    write_test_header,
    step,
    write_test_footer,
    check_address_types,
    reset_config_on_routers,
    check_router_status,
)
from lib.topolog import logger
from lib.bgp import (
    verify_bgp_convergence,
    create_router_bgp,
    clear_bgp_and_verify,
)

# Global variables
topo = None
bgp_convergence = False
NETWORK = {
    "ipv4": [
        "192.168.20.1/32",
        "192.168.20.2/32",
        "192.168.21.1/32",
        "192.168.21.2/32",
        "192.168.22.1/32",
        "192.168.22.2/32",
    ],
    "ipv6": [
        "fc07:50::1/128",
        "fc07:50::2/128",
        "fc07:150::1/128",
        "fc07:150::2/128",
        "fc07:1::1/128",
        "fc07:1::2/128",
    ],
}

bgp_convergence = False
ADDR_TYPES = check_address_types()


def setup_module(mod):
    """setup_module.

    Set up the pytest environment
    * `mod`: module name
    """
    global topo
    testsuite_run_time = time.asctime(time.localtime(time.time()))
    logger.info("Testsuite start time: {}".format(testsuite_run_time))
    logger.info("=" * 40)

    logger.info("Running setup_module to create topology")

    # This function initiates the topology build with Topogen...
    json_file = "{}/bgp_unique_rid_vrf.json".format(CWD)
    tgen = Topogen(json_file, mod.__name__)
    global topo
    topo = tgen.json_topo
    # ... and here it calls Mininet initialization functions.

    # Starting topology, create tmp files which are loaded to routers
    #  to start daemons and then start routers
    start_topology(tgen)

    # Creating configuration from JSON
    build_config_from_json(tgen, topo)

    # Checking BGP convergence
    global bgp_convergence
    global ADDR_TYPES

    # Don't run this test if we have any failure.
    if tgen.routers_have_failure():
        pytest.skip(tgen.errors)

    # Api call verify whether BGP is converged
    bgp_convergence = verify_bgp_convergence(tgen, topo)
    assert bgp_convergence is True, "setup_module :Failed \n Error:" " {}".format(
        bgp_convergence
    )
    logger.info("Running setup_module() done")


def teardown_module():
    """Teardown the pytest environment"""

    logger.info("Running teardown_module to delete topology")

    tgen = get_topogen()

    # Stop toplogy and Remove tmp files
    tgen.stop_topology()

    logger.info(
        "Testsuite end time: {}".format(time.asctime(time.localtime(time.time())))
    )
    logger.info("=" * 40)


#####################################################
# Tests starting
#####################################################


def test_bgp_unique_rid_ebgp_vrf_p0():
    """
    TC: 1
    Verify iBGP session when same and different router ID is configured in user VRF(GREEN).
    """
    tgen = get_topogen()
    global bgp_convergence

    if bgp_convergence is not True:
        pytest.skip("skipped because of BGP Convergence failure")

    # test case name
    tc_name = inspect.stack()[0][3]
    write_test_header(tc_name)
    if tgen.routers_have_failure():
        check_router_status(tgen)

    step("Configure base config as per the topology")
    reset_config_on_routers(tgen)

    step(
        "Base config should be up, verify using BGP convergence on all \
    the routers for IPv4 and IPv6 nbrs"
    )

    result = verify_bgp_convergence(tgen, topo)
    assert result is True, "Testcase {} :Failed \n Error: {}".format(tc_name, result)

    step("Configure the same router id between R4 and R3 10.10.10.10")
    input_dict = {
        "r3": {"bgp": {"router_id": "10.10.10.10", "local_as": 100, "vrf": "RED"}},
        "r4": {"bgp": {"router_id": "10.10.10.10", "local_as": 200, "vrf": "RED"}},
    }
    result = create_router_bgp(tgen, topo, input_dict)
    assert result is True, "Testcase {} :Failed \n Error: {}".format(tc_name, result)

    step("Verify neighbours are in ESTAB state.")
    result = verify_bgp_convergence(tgen, topo)
    assert result is True, "Testcase {} :Failed \n Error: {}".format(tc_name, result)

    step("Configure the same router id between R5 and R3 (10.10.10.10)")
    input_dict = {
        "r3": {"bgp": {"router_id": "10.10.10.10", "local_as": 100, "vrf": "RED"}},
        "r5": {"bgp": {"router_id": "10.10.10.10", "local_as": 300, "vrf": "RED"}},
    }
    result = create_router_bgp(tgen, topo, input_dict)
    assert result is True, "Testcase {} :Failed \n Error: {}".format(tc_name, result)

    step("Verify neighbours are in ESTAB state.")
    result = verify_bgp_convergence(tgen, topo)
    assert result is True, "Testcase {} :Failed \n Error: {}".format(tc_name, result)

    step("modify the router id on r3 to different router id (11.11.11.11)")
    input_dict = {
        "r3": {"bgp": {"router_id": "11.11.11.11", "local_as": 100, "vrf": "RED"}}
    }
    result = create_router_bgp(tgen, topo, input_dict)
    assert result is True, "Testcase {} :Failed \n Error: {}".format(tc_name, result)

    step("Verify neighbours are in ESTAB state.")
    result = verify_bgp_convergence(tgen, topo)
    assert result is True, "Testcase {} :Failed \n Error: {}".format(tc_name, result)

    step("Reset bgp process")
    step("Verify neighbours are in ESTAB state.")
    dut = "r3"
    result = clear_bgp_and_verify(tgen, topo, router="r3")
    assert result is True, "Testcase {} : Failed \n Error: {}".format(tc_name, result)

    step("Clear ip bgp process with *")
    step("Verify neighbours are in ESTAB state.")
    result = clear_bgp_and_verify(tgen, topo, dut)
    assert result is True, "Testcase {} : Failed \n Error: {}".format(tc_name, result)

    step("Configure neighbours between R3 and R4 in EVPN address family.")
    input_dict = {
        "r3": {
            "bgp": {
                "local_as": 100,
                "vrf": "RED",
                "address_family": {
                    "l2vpn": {
                        "evpn": {
                            "advertise": {
                                "ipv4": {"unicast": {}},
                                "ipv6": {"unicast": {}},
                            }
                        }
                    }
                },
            }
        },
        "r4": {
            "bgp": {
                "local_as": 200,
                "vrf": "RED",
                "address_family": {
                    "l2vpn": {
                        "evpn": {
                            "advertise": {
                                "ipv4": {"unicast": {}},
                                "ipv6": {"unicast": {}},
                            }
                        }
                    }
                },
            }
        },
    }
    result = create_router_bgp(tgen, topo, input_dict)
    assert result is True, "Testcase {} :Failed \n Error: {}".format(tc_name, result)

    step("Verify neighbours are in ESTAB state.")
    result = clear_bgp_and_verify(tgen, topo, dut)
    assert result is True, "Testcase {} : Failed \n Error: {}".format(tc_name, result)

    write_test_footer(tc_name)


def test_bgp_unique_rid_ibgp_vrf_p0():
    """
    TC: 2
    Verify eBGP session when same and different router ID is configured in user vrf (VRF RED)
    """
    tgen = get_topogen()
    global bgp_convergence

    if bgp_convergence is not True:
        pytest.skip("skipped because of BGP Convergence failure")

    # test case name
    tc_name = inspect.stack()[0][3]
    write_test_header(tc_name)
    if tgen.routers_have_failure():
        check_router_status(tgen)

    step("Configure base config as per the topology")
    reset_config_on_routers(tgen)

    step(
        "Base config should be up, verify using BGP convergence on all \
    the routers for IPv4 and IPv6 nbrs"
    )

    result = verify_bgp_convergence(tgen, topo)
    assert result is True, "Testcase {} :Failed \n Error: {}".format(tc_name, result)

    step("Configure the same router id between R1 and R3 (10.10.10.10)")
    input_dict = {
        "r3": {"bgp": {"router_id": "10.10.10.10", "local_as": 100, "vrf": "RED"}},
        "r1": {"bgp": {"router_id": "10.10.10.10", "local_as": 100, "vrf": "RED"}},
    }
    result = create_router_bgp(tgen, topo, input_dict)
    assert result is True, "Testcase {} :Failed \n Error: {}".format(tc_name, result)

    step("Verify neighbours are in ESTAB state.")
    result = verify_bgp_convergence(tgen, topo)
    assert result is True, "Testcase {} :Failed \n Error: {}".format(tc_name, result)

    step("Configure the same router id between R2 and R3 (10.10.10.10)")
    input_dict = {
        "r3": {"bgp": {"router_id": "10.10.10.10", "local_as": 100, "vrf": "RED"}},
        "r2": {"bgp": {"router_id": "10.10.10.10", "local_as": 100, "vrf": "RED"}},
    }
    result = create_router_bgp(tgen, topo, input_dict)
    assert result is True, "Testcase {} :Failed \n Error: {}".format(tc_name, result)

    step("Verify neighbours are in ESTAB state.")
    result = verify_bgp_convergence(tgen, topo)
    assert result is True, "Testcase {} :Failed \n Error: {}".format(tc_name, result)

    step("modify the router id on r3 to different router id (11.11.11.11)")
    input_dict = {
        "r3": {"bgp": {"router_id": "11.11.11.11", "local_as": 100, "vrf": "RED"}}
    }
    result = create_router_bgp(tgen, topo, input_dict)
    assert result is True, "Testcase {} :Failed \n Error: {}".format(tc_name, result)

    step("Verify neighbours are in ESTAB state.")
    result = verify_bgp_convergence(tgen, topo, dut="r3")
    assert result is True, "Testcase {} :Failed \n Error: {}".format(tc_name, result)

    step("Reset bgp process")
    step("Verify neighbours are in ESTAB state.")
    dut = "r3"
    result = clear_bgp_and_verify(tgen, topo, dut)
    assert result is True, "Testcase {} : Failed \n Error: {}".format(tc_name, result)

    step("Clear ip bgp process with *")
    result = clear_bgp_and_verify(tgen, topo, dut)
    assert result is True, "Testcase {} : Failed \n Error: {}".format(tc_name, result)

    write_test_footer(tc_name)


def test_bgp_unique_rid_multi_bgp_nbrs_vrf_p0():
    """
    TC: 3
    Verify two different eBGP sessions initiated with same router ID in user VRf (RED and GREEN)

    """
    tgen = get_topogen()
    global bgp_convergence, topo

    if bgp_convergence is not True:
        pytest.skip("skipped because of BGP Convergence failure")

    # test case name
    tc_name = inspect.stack()[0][3]
    write_test_header(tc_name)
    if tgen.routers_have_failure():
        check_router_status(tgen)

    step("Configure base config as per the topology")
    reset_config_on_routers(tgen)

    step(
        "Base config should be up, verify using BGP convergence on all \
    the routers for IPv4 and IPv6 nbrs"
    )

    result = verify_bgp_convergence(tgen, topo)
    assert result is True, "Testcase {} :Failed \n Error: {}".format(tc_name, result)

    step("Configure the same router id between R3, R4 and R5 (10.10.10.10)")
    input_dict = {
        "r3": {"bgp": {"router_id": "10.10.10.10", "local_as": 100, "vrf": "RED"}},
        "r4": {"bgp": {"router_id": "10.10.10.10", "local_as": 200, "vrf": "RED"}},
        "r5": {"bgp": {"router_id": "10.10.10.10", "local_as": 300, "vrf": "RED"}},
    }
    result = create_router_bgp(tgen, topo, input_dict)
    assert result is True, "Testcase {} :Failed \n Error: {}".format(tc_name, result)

    step("Verify neighbours are in ESTAB state.")
    result = verify_bgp_convergence(tgen, topo)
    assert result is True, "Testcase {} :Failed \n Error: {}".format(tc_name, result)

    step(
        "Configure the same IP address on on R4 and R5 loopback address and \
            change the neighborship to loopback neighbours between R3 to R4 \
            and R3 to R5 respectively."
    )

    topo1 = deepcopy(topo)

    for rtr in ["r4", "r5"]:
        topo1["routers"][rtr]["links"]["lo"]["ipv4"] = "192.168.1.1/32"

    topo1["routers"]["r3"]["links"]["lo"]["ipv4"] = "192.168.1.3/32"
    build_config_from_json(tgen, topo1, save_bkup=False)

    step(
        "change the neighborship to loopback neighbours between R3 to R4 and R3 to R5 respectively."
    )
    for rtr in ["r4", "r5"]:
        configure_bgp_on_rtr = {
            "r3": {
                "bgp": {
                    "local_as": 100,
                    "vrf": "RED",
                    "address_family": {
                        "ipv4": {
                            "unicast": {"neighbor": {rtr: {"dest_link": {"lo": {}}}}}
                        }
                    },
                },
            }
        }
        result = create_router_bgp(tgen, topo1, configure_bgp_on_rtr)
        assert result is True, "Testcase {} : Failed \n Error: {}".format(
            tc_name, result
        )

    bgp_convergence = verify_bgp_convergence(tgen, topo1, dut="r3")
    assert bgp_convergence is True, "Testcase {} : Failed \n Error: {}".format(
        tc_name, bgp_convergence
    )

    step("Change the IP address on the R4 loopback.")
    topo1["routers"]["r4"]["links"]["lo"]["ipv4"] = "192.168.1.4/32"
    build_config_from_json(tgen, topo1, save_bkup=False)

    step("Verify neighbours should be again in ESTAB state. (show ip bgp neighbours)")
    bgp_convergence = verify_bgp_convergence(tgen, topo1, dut="r3")
    assert bgp_convergence is True, "Testcase {} : Failed \n Error: {}".format(
        tc_name, bgp_convergence
    )

    step("Clear ip bgp process with *")
    result = clear_bgp_and_verify(tgen, topo, router="r3")
    assert result is True, "Testcase {} : Failed \n Error: {}".format(tc_name, result)

    write_test_footer(tc_name)


if __name__ == "__main__":
    args = ["-s"] + sys.argv[1:]
    sys.exit(pytest.main(args))



#!/usr/bin/env python
# SPDX-License-Identifier: ISC

#
# test_route_scale1.py
#
# Copyright (c) 2024 by
# Nvidia, Inc.
# Donald Sharp
#

"""
test_fpm_topo1.py: Testing FPM module

"""
import os
import sys
import pytest
import json
from functools import partial

# Save the Current Working Directory to find configuration files.
CWD = os.path.dirname(os.path.realpath(__file__))
sys.path.append(os.path.join(CWD, "../"))

# pylint: disable=C0413
# Import topogen and topotest helpers
from lib import topotest
from lib.topogen import Topogen, TopoRouter, get_topogen


pytestmark = [pytest.mark.fpm, pytest.mark.sharpd]


def build_topo(tgen):
    "Build function"

    # Populate routers
    tgen.add_router("r1")

    switch = tgen.add_switch("sw1")
    switch.add_link(tgen.gears["r1"])


def setup_module(module):
    "Setup topology"

    # fpm_stub = os.system("which fpm-stub")
    # if fpm-stub:
    #    pytest.skip("")

    tgen = Topogen(build_topo, module.__name__)
    tgen.start_topology()

    router_list = tgen.routers()
    for rname, router in router_list.items():
        router.load_config(
            TopoRouter.RD_ZEBRA,
            os.path.join(CWD, "{}/zebra.conf".format(rname)),
            "-M dplane_fpm_nl",
        )
        router.load_config(
            TopoRouter.RD_SHARP, os.path.join(CWD, "{}/sharpd.conf".format(rname))
        )
        router.load_config(
            TopoRouter.RD_FPM_LISTENER,
            os.path.join(CWD, "{}/fpm_stub.conf".format(rname)),
        )

    tgen.start_router()


def teardown_module(_mod):
    "Teardown the pytest environment"

    tgen = get_topogen()

    # This function tears down the whole topology.
    tgen.stop_topology()


def test_fpm_connection_made():
    "Test that the fpm starts up and a connection is made"

    tgen = get_topogen()
    router = tgen.gears["r1"]

    fpm_counters = "{}/r1/fpm_counters.json".format(CWD)
    expected = json.loads(open(fpm_counters).read())

    test_func = partial(
        topotest.router_json_cmp, router, "show fpm status json", expected
    )

    success, result = topotest.run_and_expect(test_func, None, 30, 1)
    assert success, "Unable to connect to the fpm:\n{}".format(result)


def test_fpm_install_routes():
    "Test that simple routes installed appears to work"

    tgen = get_topogen()
    router = tgen.gears["r1"]

    # Let's install 10000 routes
    router.vtysh_cmd("sharp install routes 10.0.0.0 nexthop 192.168.44.33 10000")
    routes_file = "{}/r1/routes_summ.json".format(CWD)
    expected = json.loads(open(routes_file).read())

    test_func = partial(
        topotest.router_json_cmp, router, "show ip route summ json", expected
    )

    success, result = topotest.run_and_expect(test_func, None, 60, 1)
    assert success, "Unable to successfully install 10000 routes: {}".format(result)

    # Let's remove 10000 routes
    router.vtysh_cmd("sharp remove routes 10.0.0.0 10000")

    routes_file_removed = "{}/r1/routes_summ_removed.json".format(CWD)
    expected = json.loads(open(routes_file_removed).read())

    test_func = partial(
        topotest.router_json_cmp, router, "show ip route summ json", expected
    )

    success, result = topotest.run_and_expect(test_func, None, 60, 1)
    assert success, "Unable to remove 10000 routes: {}".format(result)


if __name__ == "__main__":
    args = ["-s"] + sys.argv[1:]
    sys.exit(pytest.main(args))



#!/usr/bin/env python
# SPDX-License-Identifier: ISC

#
# test_bgp_route_map_on_match_next.py
#
# Copyright (c) 2023 Rubicon Communications, LLC.
#

"""
Test whether `on-match next` added to an existing route-map entry takes effect.
"""

import os
import sys
import json
import pytest
import functools

CWD = os.path.dirname(os.path.realpath(__file__))
sys.path.append(os.path.join(CWD, "../"))

# pylint: disable=C0413
from lib import topotest
from lib.topogen import Topogen, TopoRouter, get_topogen

pytestmark = [pytest.mark.bgpd]


def build_topo(tgen):
    for routern in range(1, 3):
        tgen.add_router("r{}".format(routern))

    switch = tgen.add_switch("s1")
    switch.add_link(tgen.gears["r1"])
    switch.add_link(tgen.gears["r2"])


def setup_module(mod):
    tgen = Topogen(build_topo, mod.__name__)
    tgen.start_topology()

    router_list = tgen.routers()

    for _, (rname, router) in enumerate(router_list.items(), 1):
        router.load_config(
            TopoRouter.RD_ZEBRA, os.path.join(CWD, "{}/zebra.conf".format(rname))
        )
        router.load_config(
            TopoRouter.RD_BGP, os.path.join(CWD, "{}/bgpd.conf".format(rname))
        )

    tgen.start_router()


def teardown_module(mod):
    tgen = get_topogen()
    tgen.stop_topology()


def test_bgp_route_map_on_match_next():
    tgen = get_topogen()

    if tgen.routers_have_failure():
        pytest.skip(tgen.errors)

    router1 = tgen.gears["r1"]
    router2 = tgen.gears["r2"]

    def _bgp_converge(router):
        output = json.loads(router.vtysh_cmd("show ip bgp neighbor 192.168.255.1 json"))
        expected = {"192.168.255.1": {"bgpState": "Established"}}
        return topotest.json_cmp(output, expected)

    def _bgp_has_routes(router, metric, weight):
        output = json.loads(
            router.vtysh_cmd("show ip bgp neighbor 192.168.255.1 routes json")
        )
        expected = {
            "routes": {"10.100.100.1/32": [{"metric": metric, "weight": weight}]}
        }
        return topotest.json_cmp(output, expected)

    # Check thst session is established
    test_func = functools.partial(_bgp_converge, router2)
    _, result = topotest.run_and_expect(test_func, None, count=30, wait=0.5)
    assert result is None, "Failed bgp convergence on r2"

    # Check that metric is 0 and weight is 100 for the received prefix
    test_func = functools.partial(_bgp_has_routes, router2, 0, 100)
    _, result = topotest.run_and_expect(test_func, None, count=30, wait=0.5)
    assert result is None, "r2 does not receive routes with metric 0 and weight 100"

    # Update the route-map and add "on-match next" to entry 10
    cmd = """
        configure terminal
        route-map RM permit 10
          on-match next
        exit
    """
    router2.vtysh_cmd(cmd)

    # Check that metric is 20 and weight is 100 for the received prefix
    test_func = functools.partial(_bgp_has_routes, router2, 20, 100)
    _, result = topotest.run_and_expect(test_func, None, count=30, wait=0.5)
    assert result is None, "r2 does not receive routes with metric 20 and weight 100"


if __name__ == "__main__":
    args = ["-s"] + sys.argv[1:]
    sys.exit(pytest.main(args))



#!/usr/bin/env python
# SPDX-License-Identifier: ISC

#
# test_srv6_encap_src_addr.py
# Part of NetDEF Topology Tests
#
# Copyright (c) 2022 by
# University of Rome Tor Vergata, Carmine Scarpitta <carmine.scarpitta@uniroma2.it>
#

"""
test_srv6_encap_src_addr.py:
Test for SRv6 encap source address on zebra
"""

import os
import sys
import json
import pytest

CWD = os.path.dirname(os.path.realpath(__file__))
sys.path.append(os.path.join(CWD, "../"))

# pylint: disable=C0413
from lib import topotest
from lib.topogen import Topogen, TopoRouter, get_topogen
from lib.topolog import logger

pytestmark = [pytest.mark.bgpd, pytest.mark.sharpd]


def open_json_file(filename):
    try:
        with open(filename, "r") as f:
            return json.load(f)
    except IOError:
        assert False, "Could not read file {}".format(filename)


def setup_module(mod):
    tgen = Topogen({None: "r1"}, mod.__name__)
    tgen.start_topology()
    for rname, router in tgen.routers().items():
        router.run("/bin/bash {}/{}/setup.sh".format(CWD, rname))
        router.load_config(
            TopoRouter.RD_ZEBRA, os.path.join(CWD, "{}/zebra.conf".format(rname))
        )
        router.load_config(
            TopoRouter.RD_BGP, os.path.join(CWD, "{}/bgpd.conf".format(rname))
        )
        router.load_config(
            TopoRouter.RD_SHARP, os.path.join(CWD, "{}/sharpd.conf".format(rname))
        )
    tgen.start_router()


def teardown_module():
    tgen = get_topogen()
    tgen.stop_topology()


def test_zebra_srv6_encap_src_addr(tgen):
    "Test SRv6 encapsulation source address."
    logger.info("Test SRv6 encapsulation source address.")
    r1 = tgen.gears["r1"]

    # Generate expected results
    json_file = "{}/r1/expected_srv6_encap_src_addr.json".format(CWD)
    expected = json.loads(open(json_file).read())

    ok = topotest.router_json_cmp_retry(
        r1, "show segment-routing srv6 manager json", expected
    )
    assert ok, '"r1" JSON output mismatches'

    output = r1.cmd("ip sr tunsrc show")
    assert output == "tunsrc addr fc00:0:1::1\n"


def test_zebra_srv6_encap_src_addr_unset(tgen):
    "Test SRv6 encapsulation source address unset."
    logger.info("Test SRv6 encapsulation source address unset.")
    r1 = tgen.gears["r1"]

    # Unset SRv6 encapsulation source address
    r1.vtysh_cmd(
        """
        configure terminal
         segment-routing
          srv6
           encapsulation
            no source-address
        """
    )

    # Generate expected results
    json_file = "{}/r1/expected_srv6_encap_src_addr_unset.json".format(CWD)
    expected = json.loads(open(json_file).read())

    ok = topotest.router_json_cmp_retry(
        r1, "show segment-routing srv6 manager json", expected
    )
    assert ok, '"r1" JSON output mismatches'

    output = r1.cmd("ip sr tunsrc show")
    assert output == "tunsrc addr ::\n"


def test_zebra_srv6_encap_src_addr_set(tgen):
    "Test SRv6 encapsulation source address set."
    logger.info("Test SRv6 encapsulation source address set.")
    r1 = tgen.gears["r1"]

    # Set SRv6 encapsulation source address
    r1.vtysh_cmd(
        """
        configure terminal
         segment-routing
          srv6
           encapsulation
            source-address fc00:0:1::1
        """
    )

    # Generate expected results
    json_file = "{}/r1/expected_srv6_encap_src_addr_set.json".format(CWD)
    expected = json.loads(open(json_file).read())

    ok = topotest.router_json_cmp_retry(
        r1, "show segment-routing srv6 manager json", expected
    )
    assert ok, '"r1" JSON output mismatches'

    output = r1.cmd("ip sr tunsrc show")
    assert output == "tunsrc addr fc00:0:1::1\n"


if __name__ == "__main__":
    args = ["-s"] + sys.argv[1:]
    sys.exit(pytest.main(args))



#!/usr/bin/env python
# SPDX-License-Identifier: ISC

#
# Copyright (c) 2020 by VMware, Inc. ("VMware")
# Used Copyright (c) 2018 by Network Device Education Foundation,
# Inc. ("NetDEF") in this file.
#

"""
Following tests are covered to test EVPN-Type5 functionality:

1. RD verification (manual/auto).
2. RT verification(manual)
3. In an active/standby EVPN implementation, if active DCG goes down,
        secondary takes over.
4. EVPN routes are advertised/withdrawn, based on VNFs
        advertising/withdrawing IP prefixes.
5. Route-map operations for EVPN address family.
6. BGP attributes for EVPN address-family.
"""

import os
import sys
import time
import pytest
import platform
import functools
from lib import topotest
from copy import deepcopy


# Save the Current Working Directory to find configuration files.
CWD = os.path.dirname(os.path.realpath(__file__))
sys.path.append(os.path.join(CWD, "../"))
sys.path.append(os.path.join(CWD, "../lib/"))

# Required to instantiate the topology builder class.

# pylint: disable=C0413
# Import topogen and topotest helpers
from lib.topotest import version_cmp
from lib.topogen import Topogen, get_topogen

from lib.common_config import (
    start_topology,
    write_test_header,
    check_address_types,
    write_test_footer,
    reset_config_on_routers,
    verify_rib,
    step,
    create_route_maps,
    create_static_routes,
    create_vrf_cfg,
    check_router_status,
    apply_raw_config,
    configure_vxlan,
    configure_brctl,
    create_interface_in_kernel,
    kill_router_daemons,
    start_router_daemons,
)

from lib.topolog import logger
from lib.bgp import (
    verify_bgp_convergence,
    create_router_bgp,
    verify_best_path_as_per_bgp_attribute,
    verify_attributes_for_evpn_routes,
    verify_evpn_routes,
)
from lib.topojson import build_config_from_json

pytestmark = [pytest.mark.bgpd, pytest.mark.staticd]

# Global variables
NETWORK1_1 = {"ipv4": "10.1.1.1/32", "ipv6": "10::1/128"}
NETWORK1_2 = {"ipv4": "40.1.1.1/32", "ipv6": "40::1/128"}
NETWORK1_3 = {"ipv4": "40.1.1.2/32", "ipv6": "40::2/128"}
NETWORK1_4 = {"ipv4": "40.1.1.3/32", "ipv6": "40::3/128"}
NETWORK2_1 = {"ipv4": "20.1.1.1/32", "ipv6": "20::1/128"}
NETWORK3_1 = {"ipv4": "30.1.1.1/32", "ipv6": "30::1/128"}
NETWORK4_1 = {"ipv4": "100.1.1.1/32 ", "ipv6": "100::100/128"}
NEXT_HOP_IP = {"ipv4": "Null0", "ipv6": "Null0"}
VNI_1 = 75100
VNI_2 = 75200
VNI_3 = 75300
MAC_1 = "00:80:48:ba:d1:00"
MAC_2 = "00:80:48:ba:d1:01"
MAC_3 = "00:80:48:ba:d1:02"
BRCTL_1 = "br100"
BRCTL_2 = "br200"
BRCTL_3 = "br300"
VXLAN_1 = "vxlan75100"
VXLAN_2 = "vxlan75200"
VXLAN_3 = "vxlan75300"
BRIDGE_INTF1 = "120.0.0.1"
BRIDGE_INTF2 = "120.0.0.2"
BRIDGE_INTF3 = "120.0.0.3"

VXLAN = {
    "vxlan_name": [VXLAN_1, VXLAN_2, VXLAN_3],
    "vxlan_id": [75100, 75200, 75300],
    "dstport": 4789,
    "local_addr": {"e1": BRIDGE_INTF1, "d1": BRIDGE_INTF2, "d2": BRIDGE_INTF3},
    "learning": "no",
}
BRCTL = {
    "brctl_name": [BRCTL_1, BRCTL_2, BRCTL_3],
    "addvxlan": [VXLAN_1, VXLAN_2, VXLAN_3],
    "vrf": ["RED", "BLUE", "GREEN"],
    "stp": [0, 0, 0],
}


def setup_module(mod):
    """
    Sets up the pytest environment

    * `mod`: module name
    """

    global topo
    testsuite_run_time = time.asctime(time.localtime(time.time()))
    logger.info("Testsuite start time: {}".format(testsuite_run_time))
    logger.info("=" * 40)

    logger.info("Running setup_module to create topology")

    # This function initiates the topology build with Topogen...
    json_file = "{}/evpn_type5_topo1.json".format(CWD)
    tgen = Topogen(json_file, mod.__name__)
    topo = tgen.json_topo

    # ... and here it calls Mininet initialization functions.

    # Starting topology, create tmp files which are loaded to routers
    #  to start daemons and then start routers
    start_topology(tgen)

    # Creating configuration from JSON
    build_config_from_json(tgen, topo)

    if version_cmp(platform.release(), "4.19") < 0:
        error_msg = (
            'EVPN tests will not run (have kernel "{}", '
            "but it requires >= 4.19)".format(platform.release())
        )
        pytest.skip(error_msg)

    global BGP_CONVERGENCE
    global ADDR_TYPES
    ADDR_TYPES = check_address_types()

    BGP_CONVERGENCE = verify_bgp_convergence(tgen, topo)
    assert BGP_CONVERGENCE is True, "setup_module :Failed \n Error: {}".format(
        BGP_CONVERGENCE
    )

    logger.info("Pre-requisite config for testsuite")
    prerequisite_config_for_test_suite(tgen)

    logger.info("Running setup_module() done")


def teardown_module():
    """Teardown the pytest environment"""

    logger.info("Running teardown_module to delete topology")

    tgen = get_topogen()

    # Stop toplogy and Remove tmp files
    tgen.stop_topology()

    logger.info(
        "Testsuite end time: {}".format(time.asctime(time.localtime(time.time())))
    )
    logger.info("=" * 40)


#####################################################
#
#   Testcases
#
#####################################################


def prerequisite_config_for_test_suite(tgen):
    """
    API to do prerequisite config for testsuite

    parameters:
    -----------
    * `tgen`: topogen object
    """

    step("Configure vxlan, bridge interface")
    for dut in ["e1", "d1", "d2"]:
        step("[DUT: ]Configure vxlan")
        vxlan_input = {
            dut: {
                "vxlan": [
                    {
                        "vxlan_name": VXLAN["vxlan_name"],
                        "vxlan_id": VXLAN["vxlan_id"],
                        "dstport": VXLAN["dstport"],
                        "local_addr": VXLAN["local_addr"][dut],
                        "learning": VXLAN["learning"],
                    }
                ]
            }
        }

        result = configure_vxlan(tgen, vxlan_input)
        assert result is True, "Testcase :Failed \n Error: {}".format(result)

        step("Configure bridge interface")
        brctl_input = {
            dut: {
                "brctl": [
                    {
                        "brctl_name": BRCTL["brctl_name"],
                        "addvxlan": BRCTL["addvxlan"],
                        "vrf": BRCTL["vrf"],
                        "stp": BRCTL["stp"],
                    }
                ]
            }
        }
        result = configure_brctl(tgen, topo, brctl_input)
        assert result is True, "Testcase :Failed \n Error: {}".format(result)

    step("Configure default routes")
    add_default_routes(tgen)


def add_default_routes(tgen):
    """
    API to do prerequisite config for testsuite

    parameters:
    -----------
    * `tgen`: topogen object
    """

    step("Add default routes..")

    default_routes = {
        "e1": {
            "static_routes": [
                {
                    "network": "{}/32".format(VXLAN["local_addr"]["d1"]),
                    "next_hop": topo["routers"]["d1"]["links"]["e1-link1"][
                        "ipv4"
                    ].split("/")[0],
                },
                {
                    "network": "{}/32".format(VXLAN["local_addr"]["d2"]),
                    "next_hop": topo["routers"]["d2"]["links"]["e1-link1"][
                        "ipv4"
                    ].split("/")[0],
                },
            ]
        },
        "d1": {
            "static_routes": [
                {
                    "network": "{}/32".format(VXLAN["local_addr"]["e1"]),
                    "next_hop": topo["routers"]["e1"]["links"]["d1-link1"][
                        "ipv4"
                    ].split("/")[0],
                },
                {
                    "network": "{}/32".format(VXLAN["local_addr"]["d2"]),
                    "next_hop": topo["routers"]["e1"]["links"]["d1-link1"][
                        "ipv4"
                    ].split("/")[0],
                },
            ]
        },
        "d2": {
            "static_routes": [
                {
                    "network": "{}/32".format(VXLAN["local_addr"]["d1"]),
                    "next_hop": topo["routers"]["e1"]["links"]["d2-link1"][
                        "ipv4"
                    ].split("/")[0],
                },
                {
                    "network": "{}/32".format(VXLAN["local_addr"]["e1"]),
                    "next_hop": topo["routers"]["e1"]["links"]["d2-link1"][
                        "ipv4"
                    ].split("/")[0],
                },
            ]
        },
    }

    result = create_static_routes(tgen, default_routes)
    assert result is True, "Testcase :Failed \n Error: {}".format(result)


def test_RD_verification_manual_and_auto_p0(request):
    """
    RD verification (manual/auto).
    """

    tgen = get_topogen()
    tc_name = request.node.name
    write_test_header(tc_name)
    check_router_status(tgen)
    reset_config_on_routers(tgen)
    add_default_routes(tgen)

    if tgen.routers_have_failure():
        pytest.skip(tgen.errors)

    step(
        "Advertise prefixes from VNF routers R1 and R2 in associated "
        "VRFs for both address-family."
    )
    step(
        "Advertise vrf RED's routes in EVPN address family from Edge-1 router"
        ", without manual configuration of RD."
    )

    for addr_type in ADDR_TYPES:
        input_dict_1 = {
            "r1": {
                "static_routes": [
                    {
                        "network": NETWORK1_1[addr_type],
                        "next_hop": NEXT_HOP_IP[addr_type],
                        "vrf": "RED",
                    }
                ]
            },
            "r2": {
                "static_routes": [
                    {
                        "network": NETWORK2_1[addr_type],
                        "next_hop": NEXT_HOP_IP[addr_type],
                        "vrf": "BLUE",
                    },
                    {
                        "network": NETWORK3_1[addr_type],
                        "next_hop": NEXT_HOP_IP[addr_type],
                        "vrf": "GREEN",
                    },
                ]
            },
        }

        result = create_static_routes(tgen, input_dict_1)
        assert result is True, "Testcase {} : Failed \n Error: {}".format(
            tc_name, result
        )

    step("Verify on DCG-1 and DCG-2:")
    step("EVPN route for 10.1.1.1/32 has auto-assigned RD value.")

    for dut in ["d1", "d2"]:
        input_routes = {key: topo["routers"][key] for key in ["r1"]}
        result = verify_attributes_for_evpn_routes(
            tgen, topo, dut, input_routes, rd="auto", rd_peer="e1"
        )
        assert result is True, "Testcase {} on {} :Failed \n Error: {}".format(
            tc_name, dut, result
        )

    step(
        "Configure RD for vrf RED manually as 50.50.50.50:50 and "
        "advertise vrf RED's routes in EVPN address family from "
        "Edge-1 router."
    )

    input_dict_rd = {
        "e1": {
            "bgp": [
                {
                    "local_as": "100",
                    "vrf": "RED",
                    "address_family": {"l2vpn": {"evpn": {"rd": "50.50.50.50:50"}}},
                }
            ]
        }
    }

    result = create_router_bgp(tgen, topo, input_dict_rd)
    assert result is True, "Testcase {} :Failed \n Error: {}".format(tc_name, result)

    step("EVPN route for vrf RED has RD value as 50.50.50.50:50")
    for dut in ["d1", "d2"]:
        input_routes = {key: topo["routers"][key] for key in ["r1"]}
        result = verify_attributes_for_evpn_routes(
            tgen, topo, dut, input_routes, rd="50.50.50.50:50"
        )
        assert result is True, "Testcase {} on {} :Failed \n Error: {}".format(
            tc_name, dut, result
        )

    step(
        "Configure RD for vrf RED manually as 100.100.100.100:100 and "
        "advertise vrf RED's routes in EVPN address family from Edge-1 "
        "router."
    )
    input_dict_rd = {
        "e1": {
            "bgp": [
                {
                    "local_as": "100",
                    "vrf": "RED",
                    "address_family": {
                        "l2vpn": {"evpn": {"rd": "100.100.100.100:100"}}
                    },
                }
            ]
        }
    }

    result = create_router_bgp(tgen, topo, input_dict_rd)
    assert result is True, "Testcase {} :Failed \n Error: {}".format(tc_name, result)

    step(
        "EVPN route for vrf RED is overridden with RD value as " "100.100.100.100:100."
    )

    for dut in ["d1", "d2"]:
        input_routes = {key: topo["routers"][key] for key in ["r1"]}
        result = verify_attributes_for_evpn_routes(
            tgen, topo, dut, input_routes, rd="100.100.100.100:100"
        )
        assert result is True, "Testcase {} on {} :Failed \n Error: {}".format(
            tc_name, dut, result
        )

    step(
        "Configure RD for vrf BLUE manually same as vrf RED "
        "(100.100.100.100:100) and advertise vrf RED and BLUE's routes "
        "in EVPN address family from Edge-1 router."
    )

    input_dict_rd = {
        "e1": {
            "bgp": [
                {
                    "local_as": "100",
                    "vrf": "BLUE",
                    "address_family": {
                        "l2vpn": {"evpn": {"rd": "100.100.100.100:100"}}
                    },
                }
            ]
        }
    }

    result = create_router_bgp(tgen, topo, input_dict_rd)
    assert result is True, "Testcase {} :Failed \n Error: {}".format(tc_name, result)

    step(
        "Delete manually configured RD and advertise vrf RED's routes "
        "in EVPN address family from Edge-1 router."
    )

    input_dict_rd = {
        "e1": {
            "bgp": [
                {
                    "local_as": "100",
                    "vrf": "RED",
                    "address_family": {
                        "l2vpn": {"evpn": {"no rd": "100.100.100.100:100"}}
                    },
                }
            ]
        }
    }

    result = create_router_bgp(tgen, topo, input_dict_rd)
    assert result is True, "Testcase {} :Failed \n Error: {}".format(tc_name, result)

    step(
        "Configure same RD value for vrf GREEN, as auto generated RD "
        "value for vrf RED on Edge-1 router."
    )

    input_dict_rd = {
        "e1": {
            "bgp": [
                {
                    "local_as": "100",
                    "vrf": "GREEN",
                    "address_family": {"l2vpn": {"evpn": {"rd": "10.0.0.33:1"}}},
                }
            ]
        }
    }

    result = create_router_bgp(tgen, topo, input_dict_rd)
    assert result is True, "Testcase {} :Failed \n Error: {}".format(tc_name, result)

    step("Delete auto configured RD value from vrf RED in EVPN " "address family.")

    input_dict_rd = {
        "e1": {
            "bgp": [
                {
                    "local_as": "100",
                    "vrf": "GREEN",
                    "address_family": {"l2vpn": {"evpn": {"no rd": "10.0.0.33:1"}}},
                }
            ]
        }
    }

    result = create_router_bgp(tgen, topo, input_dict_rd)
    assert result is True, "Testcase {} :Failed \n Error: {}".format(tc_name, result)

    step("Configure RD value as 100.100.100:100")

    input_dict_rd = {
        "e1": {
            "bgp": [
                {
                    "local_as": "100",
                    "vrf": "GREEN",
                    "address_family": {"l2vpn": {"evpn": {"rd": "100.100.100:100"}}},
                }
            ]
        }
    }

    result = create_router_bgp(tgen, topo, input_dict_rd)
    assert result is True, "Testcase {} :Failed \n Error: {}".format(tc_name, result)

    write_test_footer(tc_name)


def test_RT_verification_manual_p0(request):
    """
    RT verification(manual)
    """

    tgen = get_topogen()
    tc_name = request.node.name
    write_test_header(tc_name)
    check_router_status(tgen)
    reset_config_on_routers(tgen)
    add_default_routes(tgen)

    if tgen.routers_have_failure():
        pytest.skip(tgen.errors)

    step(
        "Advertise prefixes from VNF routers R1 and R2 in associated "
        "VRFs for both address-family."
    )
    step("Advertise VRF routes as in EVPN address family from Edge-1 " "router.")

    for addr_type in ADDR_TYPES:
        input_dict_1 = {
            "r1": {
                "static_routes": [
                    {
                        "network": NETWORK1_1[addr_type],
                        "next_hop": NEXT_HOP_IP[addr_type],
                        "vrf": "RED",
                    }
                ]
            },
            "r2": {
                "static_routes": [
                    {
                        "network": NETWORK2_1[addr_type],
                        "next_hop": NEXT_HOP_IP[addr_type],
                        "vrf": "BLUE",
                    },
                    {
                        "network": NETWORK3_1[addr_type],
                        "next_hop": NEXT_HOP_IP[addr_type],
                        "vrf": "GREEN",
                    },
                ]
            },
        }

        result = create_static_routes(tgen, input_dict_1)
        assert result is True, "Testcase {} : Failed \n Error: {}".format(
            tc_name, result
        )

    step(
        "Configure RT for vrf RED manually as export 100:100 "
        "and advertise vrf RED's routes in EVPN address family"
        " from Edge-1 router."
    )

    input_dict_rt = {
        "e1": {
            "bgp": [
                {
                    "local_as": "100",
                    "vrf": "RED",
                    "address_family": {
                        "ipv4": {
                            "unicast": {"neighbor": {"r1": {"dest_link": {"e1": {}}}}}
                        },
                        "ipv6": {
                            "unicast": {"neighbor": {"r1": {"dest_link": {"e1": {}}}}}
                        },
                        "l2vpn": {
                            "evpn": {"route-target": {"export": [{"value": "100:100"}]}}
                        },
                    },
                }
            ]
        }
    }

    result = create_router_bgp(tgen, topo, input_dict_rt)
    assert result is True, "Testcase {} :Failed \n Error: {}".format(tc_name, result)

    step(
        "Verify on dcg-1 and dcg-2, EVPN route for 10.1.1.1/32"
        " and 10::1/128 have RT value as 100:100."
    )

    for dut in ["d1", "d2"]:
        input_routes = {key: topo["routers"][key] for key in ["r1"]}
        result = verify_attributes_for_evpn_routes(
            tgen, topo, dut, input_routes, rt="100:100"
        )
        assert result is True, "Testcase {} on {} :Failed \n Error: {}".format(
            tc_name, dut, result
        )

    step(
        "Configure RT for vrf RED manually as export 500:500 and"
        " advertise vrf RED's routes in EVPN address family from"
        " e1 router."
    )

    input_dict_rt = {
        "e1": {
            "bgp": [
                {
                    "local_as": "100",
                    "vrf": "RED",
                    "address_family": {
                        "l2vpn": {
                            "evpn": {"route-target": {"export": [{"value": "500:500"}]}}
                        }
                    },
                }
            ]
        }
    }

    result = create_router_bgp(tgen, topo, input_dict_rt)
    assert result is True, "Testcase {} :Failed \n Error: {}".format(tc_name, result)

    step(
        "Verify on dcg-1 and dcg-2, EVPN route for 10.1.1.1/32"
        " and 10::1/128 have RT value as 500:500."
    )

    for dut in ["d1", "d2"]:
        input_routes = {key: topo["routers"][key] for key in ["r1"]}
        result = verify_attributes_for_evpn_routes(
            tgen, topo, dut, input_routes, rt=["100:100", "500:500"]
        )
        assert result is True, "Testcase {} on {} :Failed \n Error: {}".format(
            tc_name, dut, result
        )

    step(
        "Import RT value 100:100 and 500:500 in vrf BLUE manually on"
        " peer router DCG-1 and DCG-2."
    )

    input_dict_rt = {
        "d1": {
            "bgp": [
                {
                    "local_as": "100",
                    "vrf": "BLUE",
                    "address_family": {
                        "l2vpn": {
                            "evpn": {
                                "route-target": {
                                    "import": [
                                        {"value": "100:100"},
                                        {"value": "500:500"},
                                    ]
                                }
                            }
                        }
                    },
                }
            ]
        },
        "d2": {
            "bgp": [
                {
                    "local_as": "200",
                    "vrf": "BLUE",
                    "address_family": {
                        "l2vpn": {
                            "evpn": {
                                "route-target": {
                                    "import": [
                                        {"value": "100:100"},
                                        {"value": "500:500"},
                                    ]
                                }
                            }
                        }
                    },
                }
            ]
        },
    }

    result = create_router_bgp(tgen, topo, input_dict_rt)
    assert result is True, "Testcase {} :Failed \n Error: {}".format(tc_name, result)

    step(
        "EVPN route for 10.1.1.1/32 and 10::1 should be installed "
        "in vrf BLUE on DCG-1 and DCG-2 and further advertised to "
        "VNF router."
    )

    for addr_type in ADDR_TYPES:
        input_routes = {
            "r1": {
                "static_routes": [{"network": [NETWORK1_1[addr_type]], "vrf": "BLUE"}]
            }
        }
        result = verify_rib(tgen, addr_type, "d1", input_routes)
        assert result is True, "Testcase {} :Failed \n Error {}".format(tc_name, result)

        result = verify_rib(tgen, addr_type, "d2", input_routes)
        assert result is True, "Testcase {} :Failed \n Error {}".format(tc_name, result)

    step(
        "Delete import RT value 500:500 in vrf BLUE manually on "
        "peer router DCG-1 and DCG-2."
    )

    input_dict_rt = {
        "d1": {
            "bgp": [
                {
                    "local_as": "100",
                    "vrf": "BLUE",
                    "address_family": {
                        "l2vpn": {
                            "evpn": {
                                "route-target": {
                                    "import": [{"value": "500:500", "delete": True}]
                                }
                            }
                        }
                    },
                }
            ]
        },
        "d2": {
            "bgp": [
                {
                    "local_as": "200",
                    "vrf": "BLUE",
                    "address_family": {
                        "l2vpn": {
                            "evpn": {
                                "route-target": {
                                    "import": [{"value": "500:500", "delete": True}]
                                }
                            }
                        }
                    },
                }
            ]
        },
    }

    result = create_router_bgp(tgen, topo, input_dict_rt)
    assert result is True, "Testcase {} :Failed \n Error: {}".format(tc_name, result)

    for dut in ["d1", "d2"]:
        input_routes = {key: topo["routers"][key] for key in ["r1"]}
        result = verify_attributes_for_evpn_routes(
            tgen, topo, dut, input_routes, rt=["100:100", "500:500"]
        )
        assert result is True, "Testcase {} on {} :Failed \n Error: {}".format(
            tc_name, dut, result
        )

    step("Delete RT export value 100:100 for vrf RED on Edge-1")

    input_dict_rt = {
        "e1": {
            "bgp": [
                {
                    "local_as": "100",
                    "vrf": "RED",
                    "address_family": {
                        "l2vpn": {
                            "evpn": {
                                "route-target": {
                                    "export": [{"value": "100:100", "delete": True}]
                                }
                            }
                        }
                    },
                }
            ]
        }
    }

    result = create_router_bgp(tgen, topo, input_dict_rt)
    assert result is True, "Testcase {} :Failed \n Error: {}".format(tc_name, result)

    step(
        "EVPN route for 10.1.1.1/32 and 10::1 should be withdrawn "
        "from vrf BLUE on DCG-1,DCG-2 and VNF router."
    )

    for addr_type in ADDR_TYPES:
        input_routes = {
            "r1": {
                "static_routes": [{"network": [NETWORK1_1[addr_type]], "vrf": "BLUE"}]
            }
        }
        result = verify_rib(tgen, addr_type, "d1", input_routes, expected=False)
        assert result is not True, (
            "Testcase {} : Failed \n "
            "Expected: Routes should not be present in {} RIB \n "
            "Found: {}".format(tc_name, "d1", result)
        )

        result = verify_rib(tgen, addr_type, "d2", input_routes, expected=False)
        assert result is not True, (
            "Testcase {} : Failed \n "
            "Expected: Routes should not be present in {} RIB \n "
            "Found: {}".format(tc_name, "d2", result)
        )

    step(
        "Configure RT value as 100:100000010000010000101010 to check "
        "the boundary value."
    )

    input_dict_rt = {
        "e1": {
            "bgp": [
                {
                    "local_as": "100",
                    "vrf": "RED",
                    "address_family": {
                        "l2vpn": {
                            "evpn": {
                                "route-target": {
                                    "export": [
                                        {"value": "100:100000010000010000101010"}
                                    ]
                                }
                            }
                        }
                    },
                }
            ]
        }
    }

    result = create_router_bgp(tgen, topo, input_dict_rt)
    assert result is True, "Testcase {} :Failed \n Error: {}".format(tc_name, result)

    step(
        "CLI error: RT value: 100:100000010000010000101010 should not " "be configured"
    )

    dut = "e1"
    input_routes = {key: topo["routers"][key] for key in ["r1"]}
    result = verify_attributes_for_evpn_routes(
        tgen, topo, dut, input_routes, rt="100:100000010000010000101010", expected=False
    )
    assert result is not True, (
        "Testcase {} : Failed \n "
        "Expected: RT value out of boundary error in {} \n "
        "Found: {}".format(tc_name, dut, result)
    )

    write_test_footer(tc_name)


def test_active_standby_evpn_implementation_p1(request):
    """
    In an active/standby EVPN implementation, if active DCG goes down,
    secondary takes over.
    """

    tgen = get_topogen()
    tc_name = request.node.name
    write_test_header(tc_name)
    check_router_status(tgen)
    reset_config_on_routers(tgen)
    add_default_routes(tgen)

    if tgen.routers_have_failure():
        pytest.skip(tgen.errors)

    step(
        "Taken care in base config: Configure BGP neighborship for both "
        "address families(IPv4 & IPv6) between DCG-1/DCG-2 and VFN routers"
        "(R3 and R4)."
    )

    step(
        "BGP neighborships come up within defined VRFs. Please use below "
        "command: sh bgp vrf all summary"
    )

    result = verify_bgp_convergence(tgen, topo, "d1")
    assert result is True, "Testcase {} :Failed \n Error: {}".format(tc_name, result)

    result = verify_bgp_convergence(tgen, topo, "d2")
    assert result is True, "Testcase {} :Failed \n Error: {}".format(tc_name, result)

    step(
        "Advertise prefixes from VNF routers R3 and R4 in associated "
        "VRFs for both address-families."
    )

    for addr_type in ADDR_TYPES:
        input_dict_1 = {
            "r3": {
                "static_routes": [
                    {
                        "network": NETWORK1_2[addr_type],
                        "next_hop": NEXT_HOP_IP[addr_type],
                        "vrf": "RED",
                    }
                ]
            },
            "r4": {
                "static_routes": [
                    {
                        "network": NETWORK1_3[addr_type],
                        "next_hop": NEXT_HOP_IP[addr_type],
                        "vrf": "BLUE",
                    },
                    {
                        "network": NETWORK1_4[addr_type],
                        "next_hop": NEXT_HOP_IP[addr_type],
                        "vrf": "GREEN",
                    },
                ]
            },
        }

        result = create_static_routes(tgen, input_dict_1)
        assert result is True, "Testcase {} : Failed \n Error: {}".format(
            tc_name, result
        )

    step(
        "Redistribute static in (IPv4 and IPv6) address-family "
        "on Edge-1 for all VRFs."
    )

    input_dict_2 = {}
    for dut in ["r3", "r4"]:
        temp = {dut: {"bgp": []}}
        input_dict_2.update(temp)

        if dut == "r3":
            VRFS = ["RED"]
            AS_NUM = [3]
        if dut == "r4":
            VRFS = ["BLUE", "GREEN"]
            AS_NUM = [4, 4]

        for vrf, as_num in zip(VRFS, AS_NUM):
            temp[dut]["bgp"].append(
                {
                    "local_as": as_num,
                    "vrf": vrf,
                    "address_family": {
                        "ipv4": {
                            "unicast": {"redistribute": [{"redist_type": "static"}]}
                        },
                        "ipv6": {
                            "unicast": {"redistribute": [{"redist_type": "static"}]}
                        },
                    },
                }
            )

    result = create_router_bgp(tgen, topo, input_dict_2)
    assert result is True, "Testcase {} :Failed \n Error: {}".format(tc_name, result)

    step("Prefixes are received in respective VRFs on DCG-1/DCG-2.")

    for addr_type in ADDR_TYPES:
        input_routes = {
            "r3": {
                "static_routes": [
                    {
                        "network": NETWORK1_2[addr_type],
                        "next_hop": NEXT_HOP_IP[addr_type],
                        "vrf": "RED",
                    }
                ]
            },
            "r4": {
                "static_routes": [
                    {
                        "network": NETWORK1_3[addr_type],
                        "next_hop": NEXT_HOP_IP[addr_type],
                        "vrf": "BLUE",
                    },
                    {
                        "network": NETWORK1_4[addr_type],
                        "next_hop": NEXT_HOP_IP[addr_type],
                        "vrf": "GREEN",
                    },
                ]
            },
        }

        result = verify_rib(tgen, addr_type, "d1", input_routes)
        assert result is True, "Testcase {} :Failed \n Error: {}".format(
            tc_name, result
        )

        result = verify_rib(tgen, addr_type, "d2", input_routes)
        assert result is True, "Testcase {} :Failed \n Error: {}".format(
            tc_name, result
        )

    step(
        "Taken care in base config: Advertise VRF routes in EVPN "
        "address-family from DCG-1 and DCG-2 router."
    )

    step("Verify on Edge-1 that EVPN routes are installed via next-hop " "as DCG-2.")

    for addr_type in ADDR_TYPES:
        input_routes = {
            "r3": {
                "static_routes": [
                    {
                        "network": NETWORK1_2[addr_type],
                        "next_hop": NEXT_HOP_IP[addr_type],
                        "vrf": "RED",
                    }
                ]
            },
            "r4": {
                "static_routes": [
                    {
                        "network": NETWORK1_3[addr_type],
                        "next_hop": NEXT_HOP_IP[addr_type],
                        "vrf": "BLUE",
                    },
                    {
                        "network": NETWORK1_4[addr_type],
                        "next_hop": NEXT_HOP_IP[addr_type],
                        "vrf": "GREEN",
                    },
                ]
            },
        }

        if addr_type == "ipv4":
            result = verify_rib(
                tgen, addr_type, "e1", input_routes, next_hop=BRIDGE_INTF2
            )
            assert result is True, "Testcase {} :Failed \n Error: {}".format(
                tc_name, result
            )
        else:
            result = verify_rib(tgen, addr_type, "e1", input_routes)
            assert result is True, "Testcase {} :Failed \n Error: {}".format(
                tc_name, result
            )

    step(
        "Configure 'next-hop self' on DCG-1 for peer Edge-1 in EVPN " "address-family."
    )

    input_dict_3 = {
        "d1": {
            "bgp": [
                {
                    "local_as": "100",
                    "address_family": {
                        "l2vpn": {
                            "evpn": {
                                "neighbor": {
                                    "e1": {
                                        "ipv4": {"d1-link1": {"next_hop_self": True}}
                                    }
                                }
                            }
                        }
                    },
                }
            ]
        }
    }

    result = create_router_bgp(tgen, topo, input_dict_3)
    assert result is True, "Testcase {} :Failed \n Error: {}".format(tc_name, result)

    logger.info(
        "Creating route-map so ipv6 glpbal ip wpuld be preferred " "as next-hop"
    )

    step(
        "Verify on Edge-1 that EVPN routes are now preferred via "
        "next-hop as DCG-1(iBGP) due to shortest AS-Path."
    )

    for addr_type in ADDR_TYPES:
        logger.info("Verifying only ipv4 routes")
        if addr_type != "ipv4":
            continue

        input_routes = {
            "r3": {
                "static_routes": [
                    {
                        "network": NETWORK1_2[addr_type],
                        "next_hop": NEXT_HOP_IP[addr_type],
                        "vrf": "RED",
                    }
                ]
            },
            "r4": {
                "static_routes": [
                    {
                        "network": NETWORK1_3[addr_type],
                        "next_hop": NEXT_HOP_IP[addr_type],
                        "vrf": "BLUE",
                    },
                    {
                        "network": NETWORK1_4[addr_type],
                        "next_hop": NEXT_HOP_IP[addr_type],
                        "vrf": "GREEN",
                    },
                ]
            },
        }

        next_hop = topo["routers"]["d1"]["links"]["e1-link1"]["ipv4"].split("/")[0]

        result = verify_rib(tgen, addr_type, "e1", input_routes, next_hop=next_hop)
        assert result is True, "Testcase {} :Failed \n Error: {}".format(
            tc_name, result
        )

    write_test_footer(tc_name)


def test_evpn_routes_from_VNFs_p1(request):
    """
    EVPN routes are advertised/withdrawn, based on VNFs
    advertising/withdrawing IP prefixes.
    """

    tgen = get_topogen()
    tc_name = request.node.name
    write_test_header(tc_name)
    check_router_status(tgen)
    reset_config_on_routers(tgen)
    add_default_routes(tgen)

    if tgen.routers_have_failure():
        pytest.skip(tgen.errors)

    step(
        "Advertise prefixes from VNF routers R1 and R2 in associated "
        "VRFs for both address-family."
    )

    for addr_type in ADDR_TYPES:
        input_dict_1 = {
            "r1": {
                "static_routes": [
                    {
                        "network": NETWORK1_1[addr_type],
                        "next_hop": NEXT_HOP_IP[addr_type],
                        "vrf": "RED",
                    }
                ]
            },
            "r2": {
                "static_routes": [
                    {
                        "network": NETWORK2_1[addr_type],
                        "next_hop": NEXT_HOP_IP[addr_type],
                        "vrf": "BLUE",
                    },
                    {
                        "network": NETWORK3_1[addr_type],
                        "next_hop": NEXT_HOP_IP[addr_type],
                        "vrf": "GREEN",
                    },
                ]
            },
        }

        result = create_static_routes(tgen, input_dict_1)
        assert result is True, "Testcase {} : Failed \n Error: {}".format(
            tc_name, result
        )

    step(
        "Taken care in base config: Advertise VNFs'(R1 and R2) "
        "originated routes in EVPN address-family from Edge-1 to "
        "DCG-1 and DCG-2 routers."
    )
    step(
        "Taken care in base config: Advertise IPv4 and IPv6 routes "
        "from default vrf in EVPN address-family from Edge-1."
    )

    step(
        "Verify on DCG-2 that VNF routes are received in respective "
        "VRFs along with auto derived RD/RT values 'show bgp l2vpn evpn'"
    )
    for dut in ["d1", "d2"]:
        input_routes = {key: topo["routers"][key] for key in ["r1"]}
        result = verify_evpn_routes(tgen, topo, dut, input_routes)
        assert result is True, "Testcase {} on {} :Failed \n Error: {}".format(
            tc_name, dut, result
        )

        input_routes = {key: topo["routers"][key] for key in ["r2"]}
        result = verify_evpn_routes(tgen, topo, dut, input_routes)
        assert result is True, "Testcase {} on {} :Failed \n Error: {}".format(
            tc_name, dut, result
        )

    step(
        "Verify on R3 and R4 that DCG-2 further advertises all EVPN "
        "routes to corresponding VRFs."
    )
    for addr_type in ADDR_TYPES:
        input_routes = {key: topo["routers"][key] for key in ["r1"]}
        result = verify_rib(tgen, addr_type, "r3", input_routes)
        assert result is True, "Testcase {} :Failed \n Error: {}".format(
            tc_name, result
        )

    for addr_type in ADDR_TYPES:
        input_routes = {key: topo["routers"][key] for key in ["r2"]}
        result = verify_rib(tgen, addr_type, "r4", input_routes)
        assert result is True, "Testcase {} :Failed \n Error: {}".format(
            tc_name, result
        )

    step(
        "Verify that DCG-2 receives EVPN routes associated to default "
        "VRF and install in default IP routing table as well."
    )
    for addr_type in ADDR_TYPES:
        input_routes = {key: topo["routers"][key] for key in ["r1"]}
        result = verify_rib(tgen, addr_type, "d2", input_routes)
        assert result is True, "Testcase {} :Failed \n Error: {}".format(
            tc_name, result
        )

    for addr_type in ADDR_TYPES:
        input_routes = {key: topo["routers"][key] for key in ["r2"]}
        result = verify_rib(tgen, addr_type, "d2", input_routes)
        assert result is True, "Testcase {} :Failed \n Error: {}".format(
            tc_name, result
        )

    step("Withdraw the IP prefixes from VFN(R1).")
    dut = "r1"
    input_dict_2 = {}
    static_routes = topo["routers"][dut]["static_routes"]
    for static_route in static_routes:
        static_route["delete"] = True
        temp = {dut: {"static_routes": [static_route]}}
        input_dict_2.update(temp)

        result = create_static_routes(tgen, input_dict_2)
        assert result is True, "Testcase {} :Failed \n Error: {}".format(
            tc_name, result
        )

    step(
        "Verify that DCG-2 removes EVPN routes corresponding to vrf RED and "
        "send an withdraw to VNF(R3) as well."
    )
    for addr_type in ADDR_TYPES:
        input_routes = {key: topo["routers"][key] for key in ["r1"]}
        result = verify_rib(tgen, addr_type, "d2", input_routes, expected=False)
        assert result is not True, (
            "Testcase {} : Failed \n "
            "Expected: Routes should not be present in {} RIB \n "
            "Found: {}".format(tc_name, "d2", result)
        )

    for addr_type in ADDR_TYPES:
        input_routes = {key: topo["routers"][key] for key in ["r1"]}
        result = verify_rib(tgen, addr_type, "r3", input_routes, expected=False)
        assert result is not True, (
            "Testcase {} : Failed \n "
            "Expected: Routes should not be present in {} RIB \n "
            "Found: {}".format(tc_name, "r3", result)
        )

    step("Re-advertise IP prefixes from VFN(R1).")
    step(
        "Advertise prefixes from VNF routers R1 and R2 in associated "
        "VRFs for both address-family."
    )

    for addr_type in ADDR_TYPES:
        input_dict_1 = {
            "r1": {
                "static_routes": [
                    {
                        "network": NETWORK1_1[addr_type],
                        "next_hop": NEXT_HOP_IP[addr_type],
                        "vrf": "RED",
                    }
                ]
            },
            "r2": {
                "static_routes": [
                    {
                        "network": NETWORK2_1[addr_type],
                        "next_hop": NEXT_HOP_IP[addr_type],
                        "vrf": "BLUE",
                    },
                    {
                        "network": NETWORK3_1[addr_type],
                        "next_hop": NEXT_HOP_IP[addr_type],
                        "vrf": "GREEN",
                    },
                ]
            },
        }

        result = create_static_routes(tgen, input_dict_1)
        assert result is True, "Testcase {} : Failed \n Error: {}".format(
            tc_name, result
        )

    step(
        "Verify that DCG-2 receives EVPN routes corresponding to vrf RED "
        "again and send an update to VNF(R3) as well."
    )
    for addr_type in ADDR_TYPES:
        input_routes = {key: topo["routers"][key] for key in ["r1"]}
        result = verify_rib(tgen, addr_type, "d2", input_routes)
        assert result is True, "Testcase {} :Failed \n Error: {}".format(
            tc_name, result
        )

    for addr_type in ADDR_TYPES:
        input_routes = {key: topo["routers"][key] for key in ["r1"]}
        result = verify_rib(tgen, addr_type, "r3", input_routes)
        assert result is True, "Testcase {} :Failed \n Error: {}".format(
            tc_name, result
        )

    step("Delete vrf BLUE from router Edge-1")
    input_dict_3 = {"e1": {"vrfs": [{"name": "BLUE", "id": "2", "delete": True}]}}

    result = create_vrf_cfg(tgen, input_dict_3)
    assert result is True, "Testcase {} :Failed \n Error: {}".format(tc_name, result)

    step(
        "Verify that DCG-2 removes EVPN routes corresponding to "
        "vrf BLUE and send an withdraw to VNF(R4) as well."
    )
    for addr_type in ADDR_TYPES:
        input_routes = {
            "r2": {"static_routes": [{"network": NETWORK2_1[addr_type], "vrf": "BLUE"}]}
        }

        result = verify_rib(tgen, addr_type, "d2", input_routes, expected=False)
        assert result is not True, (
            "Testcase {} : Failed \n "
            "Expected: Routes should not be present in {} RIB \n "
            "Found: {}".format(tc_name, "d2", result)
        )

        result = verify_rib(tgen, addr_type, "r4", input_routes, expected=False)
        assert result is not True, (
            "Testcase {} : Failed \n "
            "Expected: Routes should not be present in {} RIB \n "
            "Found: {}".format(tc_name, "r4", result)
        )

    step("Add vrf BLUE on router Edge-1 again.")
    interface = topo["routers"]["e1"]["links"]["r2-link1"]["interface"]
    input_dict_3 = {
        "e1": {
            "links": {
                "r2-link1": {
                    "interface": interface,
                    "ipv4": "auto",
                    "ipv6": "auto",
                    "vrf": "BLUE",
                }
            },
            "vrfs": [{"name": "BLUE", "id": "2"}],
        }
    }
    result = create_vrf_cfg(tgen, input_dict_3)
    assert result is True, "Testcase {} :Failed \n Error: {}".format(tc_name, result)

    logger.info(
        "After deleting VRFs ipv6 addresses wil be deleted "
        "from kernel Adding back ipv6 addresses"
    )
    dut = "e1"
    vrfs = ["BLUE"]

    for vrf in vrfs:
        for c_link, c_data in topo["routers"][dut]["links"].items():
            if "vrf" in c_data:
                if c_data["vrf"] != vrf:
                    continue

                intf_name = c_data["interface"]
                intf_ipv6 = c_data["ipv6"]

                create_interface_in_kernel(
                    tgen, dut, intf_name, intf_ipv6, vrf, create=False
                )

    result = verify_bgp_convergence(tgen, topo, dut)
    assert result is True, "Failed to converge on {}".format(dut)

    step(
        "Verify that DCG-2 receives EVPN routes corresponding to "
        "vrf BLUE again and send an update to VNF(R4) as well."
    )
    for addr_type in ADDR_TYPES:
        input_routes = {
            "r2": {"static_routes": [{"network": NETWORK2_1[addr_type], "vrf": "BLUE"}]}
        }

        result = verify_rib(tgen, addr_type, "d2", input_routes)
        assert result is True, "Testcase {} :Failed \n Error: {}".format(
            tc_name, result
        )

        result = verify_rib(tgen, addr_type, "r4", input_routes)
        assert result is True, "Testcase {} :Failed \n Error: {}".format(
            tc_name, result
        )

    step("Withdraw IPv6 address-family in EVPN advertisements for " "VRF GREEN")
    addr_type = "ipv6"
    input_dict_4 = {
        "e1": {
            "bgp": [
                {
                    "local_as": "100",
                    "vrf": "GREEN",
                    "address_family": {
                        "l2vpn": {
                            "evpn": {
                                "advertise": {addr_type: {"unicast": {"delete": True}}}
                            }
                        }
                    },
                }
            ]
        }
    }

    result = create_router_bgp(tgen, topo, input_dict_4)
    assert result is True, "Testcase {} :Failed \n Error: {}".format(tc_name, result)

    step(
        "Verify that EVPN routes (IPv6)associated with vrf GREEN are "
        "withdrawn from DCG-2 and VNF R4."
    )
    input_routes = {
        "r2": {"static_routes": [{"network": NETWORK3_1[addr_type], "vrf": "GREEN"}]}
    }

    result = verify_rib(tgen, addr_type, "d2", input_routes, expected=False)
    assert result is not True, (
        "Testcase {} : Failed \n "
        "Expected: Routes should not be present in {} RIB \n "
        "Found: {}".format(tc_name, "d2", result)
    )

    result = verify_rib(tgen, addr_type, "r4", input_routes, expected=False)
    assert result is not True, (
        "Testcase {} : Failed \n "
        "Expected: Routes should not be present in {} RIB \n "
        "Found: {}".format(tc_name, "r4", result)
    )

    step("Advertise IPv6 address-family in EVPN advertisements " "for VRF GREEN.")
    addr_type = "ipv6"
    input_dict_4 = {
        "e1": {
            "bgp": [
                {
                    "local_as": "100",
                    "vrf": "GREEN",
                    "address_family": {
                        "l2vpn": {"evpn": {"advertise": {addr_type: {"unicast": {}}}}}
                    },
                }
            ]
        }
    }

    result = create_router_bgp(tgen, topo, input_dict_4)
    assert result is True, "Testcase {} :Failed \n Error: {}".format(tc_name, result)

    for addr_type in ADDR_TYPES:
        input_routes = {
            "r2": {
                "static_routes": [{"network": NETWORK3_1[addr_type], "vrf": "GREEN"}]
            }
        }

        result = verify_rib(tgen, addr_type, "d2", input_routes)
        assert result is True, "Testcase {} :Failed \n Error: {}".format(
            tc_name, result
        )

        result = verify_rib(tgen, addr_type, "r4", input_routes)
        assert result is True, "Testcase {} :Failed \n Error: {}".format(
            tc_name, result
        )

    write_test_footer(tc_name)


@pytest.mark.parametrize(
    "attribute", [{"route-type": "prefix"}, {"vni": VNI_1}, {"rt": "300:300"}]
)
def test_route_map_operations_for_evpn_address_family_p1(request, attribute):
    """
    Route-map operations for EVPN address family.
    """

    tgen = get_topogen()
    tc_name = request.node.name
    write_test_header(tc_name)
    check_router_status(tgen)
    reset_config_on_routers(tgen)
    add_default_routes(tgen)

    step(
        "Advertise prefixes from VNF routers R1 and R2 in associated "
        "VRFs for both address-family."
    )

    for addr_type in ADDR_TYPES:
        input_dict_1 = {
            "r1": {
                "static_routes": [
                    {
                        "network": NETWORK1_1[addr_type],
                        "next_hop": NEXT_HOP_IP[addr_type],
                        "vrf": "RED",
                    }
                ]
            },
            "r2": {
                "static_routes": [
                    {
                        "network": NETWORK2_1[addr_type],
                        "next_hop": NEXT_HOP_IP[addr_type],
                        "vrf": "BLUE",
                    },
                    {
                        "network": NETWORK3_1[addr_type],
                        "next_hop": NEXT_HOP_IP[addr_type],
                        "vrf": "GREEN",
                    },
                ]
            },
        }

        result = create_static_routes(tgen, input_dict_1)
        assert result is True, "Testcase {} : Failed \n Error: {}".format(
            tc_name, result
        )

    if tgen.routers_have_failure():
        pytest.skip(tgen.errors)

    step(
        "Advertise VRF routes in EVPN address family from Edge-1 router."
        " Configure a route-map on e1 to filter EVPN routes based on"
        " below keywords: route-type: prefix"
    )

    for key, value in attribute.items():
        if key == "rt":
            logger.info("Creating extcommunity using raw_config")
            raw_config = {
                "d2": {
                    "raw_config": [
                        "bgp extcommunity-list standard ECOMM300 permit {} {}".format(
                            key, value
                        )
                    ]
                }
            }
            result = apply_raw_config(tgen, raw_config)
            assert result is True, "Testcase {} : Failed Error: {}".format(
                tc_name, result
            )

            input_dict_1 = {
                "e1": {
                    "route_maps": {
                        "rmap_route_type": [
                            {"action": "permit", "set": {"extcommunity": {key: value}}}
                        ]
                    }
                },
                "d2": {
                    "route_maps": {
                        "rmap_route_type": [
                            {"action": "permit", "match": {"extcommunity": "ECOMM300"}}
                        ]
                    }
                },
            }

        else:
            input_dict_1 = {
                "e1": {
                    "route_maps": {
                        "rmap_route_type": [
                            {"action": "permit", "match": {"evpn": {key: value}}}
                        ]
                    }
                },
                "d2": {
                    "route_maps": {
                        "rmap_route_type": [
                            {"action": "permit", "match": {"evpn": {key: value}}}
                        ]
                    }
                },
            }
        result = create_route_maps(tgen, input_dict_1)
        assert result is True, "Testcase {} : Failed \n Error: {}".format(
            tc_name, result
        )

    input_dict_2 = {
        "e1": {
            "bgp": [
                {
                    "local_as": "100",
                    "address_family": {
                        "l2vpn": {
                            "evpn": {
                                "neighbor": {
                                    "d2": {
                                        "ipv4": {
                                            "e1-link1": {
                                                "route_maps": [
                                                    {
                                                        "name": "rmap_route_type",
                                                        "direction": "out",
                                                    }
                                                ]
                                            }
                                        }
                                    }
                                }
                            }
                        }
                    },
                }
            ]
        },
        "d2": {
            "bgp": [
                {
                    "local_as": "200",
                    "address_family": {
                        "l2vpn": {
                            "evpn": {
                                "neighbor": {
                                    "e1": {
                                        "ipv4": {
                                            "d2-link1": {
                                                "route_maps": [
                                                    {
                                                        "name": "rmap_route_type",
                                                        "direction": "in",
                                                    }
                                                ]
                                            }
                                        }
                                    }
                                }
                            }
                        }
                    },
                }
            ]
        },
    }

    result = create_router_bgp(tgen, topo, input_dict_2)
    assert result is True, "Testcase {} :Failed \n Error: {}".format(tc_name, result)

    step(
        "Verify on router DCG-2 that EVPN routes corresponding to all "
        "VRFs are received. As all EVPN routes are type-5 only."
    )

    input_routes = {key: topo["routers"][key] for key in ["r1"]}
    result = verify_evpn_routes(tgen, topo, "d2", input_routes)
    assert result is True, "Testcase {} :Failed \n Error: {}".format(tc_name, result)

    input_routes = {key: topo["routers"][key] for key in ["r2"]}
    result = verify_evpn_routes(tgen, topo, "d2", input_routes)
    assert result is True, "Testcase {} :Failed \n Error: {}".format(tc_name, result)

    write_test_footer(tc_name)


def test_evpn_address_family_with_graceful_restart_p0(request):
    """
    Verify Graceful-restart function for EVPN address-family.
    """

    tgen = get_topogen()
    tc_name = request.node.name
    write_test_header(tc_name)
    check_router_status(tgen)
    reset_config_on_routers(tgen)
    add_default_routes(tgen)

    if tgen.routers_have_failure():
        pytest.skip(tgen.errors)

    for addr_type in ADDR_TYPES:
        input_dict_1 = {
            "r3": {
                "static_routes": [
                    {
                        "network": NETWORK1_2[addr_type],
                        "next_hop": NEXT_HOP_IP[addr_type],
                        "vrf": "RED",
                    }
                ]
            },
            "r4": {
                "static_routes": [
                    {
                        "network": NETWORK1_3[addr_type],
                        "next_hop": NEXT_HOP_IP[addr_type],
                        "vrf": "BLUE",
                    },
                    {
                        "network": NETWORK1_4[addr_type],
                        "next_hop": NEXT_HOP_IP[addr_type],
                        "vrf": "GREEN",
                    },
                ]
            },
        }

        result = create_static_routes(tgen, input_dict_1)
        assert result is True, "Testcase {} : Failed \n Error: {}".format(
            tc_name, result
        )

    step(
        "Redistribute static in (IPv4 and IPv6) address-family "
        "on Edge-1 for all VRFs."
    )

    input_dict_2 = {}
    for dut in ["r3", "r4"]:
        temp = {dut: {"bgp": []}}
        input_dict_2.update(temp)

        if dut == "r3":
            VRFS = ["RED"]
            AS_NUM = [3]
        if dut == "r4":
            VRFS = ["BLUE", "GREEN"]
            AS_NUM = [4, 4]

        for vrf, as_num in zip(VRFS, AS_NUM):
            temp[dut]["bgp"].append(
                {
                    "local_as": as_num,
                    "vrf": vrf,
                    "address_family": {
                        "ipv4": {
                            "unicast": {"redistribute": [{"redist_type": "static"}]}
                        },
                        "ipv6": {
                            "unicast": {"redistribute": [{"redist_type": "static"}]}
                        },
                    },
                }
            )

    result = create_router_bgp(tgen, topo, input_dict_2)
    assert result is True, "Testcase {} :Failed \n Error: {}".format(tc_name, result)

    step(
        "Verify on router Edge-1 that EVPN routes corresponding to "
        "all VRFs are received from both routers DCG-1 and DCG-2"
    )

    for addr_type in ADDR_TYPES:
        input_routes = {
            "r3": {
                "static_routes": [
                    {
                        "network": NETWORK1_2[addr_type],
                        "next_hop": NEXT_HOP_IP[addr_type],
                        "vrf": "RED",
                    }
                ]
            },
            "r4": {
                "static_routes": [
                    {
                        "network": NETWORK1_3[addr_type],
                        "next_hop": NEXT_HOP_IP[addr_type],
                        "vrf": "BLUE",
                    },
                    {
                        "network": NETWORK1_4[addr_type],
                        "next_hop": NEXT_HOP_IP[addr_type],
                        "vrf": "GREEN",
                    },
                ]
            },
        }

        result = verify_rib(tgen, addr_type, "e1", input_routes)
        assert result is True, "Testcase {} :Failed \n Error: {}".format(
            tc_name, result
        )

    step(
        "Configure DCG-2 as GR restarting node for EVPN session between"
        " DCG-2 and EDGE-1, following by a session reset using 'clear bgp *'"
        " command."
    )

    input_dict_gr = {
        "d2": {
            "bgp": [
                {
                    "local_as": "200",
                    "graceful-restart": {
                        "graceful-restart": True,
                    },
                }
            ]
        }
    }

    result = create_router_bgp(tgen, topo, input_dict_gr)
    assert result is True, "Testcase {} : Failed \n Error: {}".format(tc_name, result)

    step(
        "Verify that DCG-2 changes it's role to GR-restarting router "
        "and EDGE-1 becomes the GR-helper."
    )

    step("Kill BGPd daemon on DCG-2.")
    kill_router_daemons(tgen, "d2", ["bgpd"])

    step(
        "Verify that EDGE-1 keep stale entries for EVPN RT-5 routes "
        "received from DCG-2 before the restart."
    )

    for addr_type in ADDR_TYPES:
        input_routes = {
            "r4": {
                "static_routes": [
                    {
                        "network": NETWORK1_3[addr_type],
                        "next_hop": NEXT_HOP_IP[addr_type],
                        "vrf": "BLUE",
                    },
                    {
                        "network": NETWORK1_4[addr_type],
                        "next_hop": NEXT_HOP_IP[addr_type],
                        "vrf": "GREEN",
                    },
                ]
            }
        }
        result = verify_evpn_routes(tgen, topo, "e1", input_routes)
        assert result is True, "Testcase {} :Failed \n Error: {}".format(
            tc_name, result
        )

    step(
        "Verify that DCG-2 keeps BGP routes in Zebra until BGPd "
        "comes up or end of 'rib-stale-time'"
    )

    step("Start BGPd daemon on DCG-2.")
    start_router_daemons(tgen, "d2", ["bgpd"])

    step("Verify that EDGE-1 removed all the stale entries.")
    for addr_type in ADDR_TYPES:
        input_routes = {
            "r4": {
                "static_routes": [
                    {
                        "network": NETWORK1_3[addr_type],
                        "next_hop": NEXT_HOP_IP[addr_type],
                        "vrf": "BLUE",
                    },
                    {
                        "network": NETWORK1_4[addr_type],
                        "next_hop": NEXT_HOP_IP[addr_type],
                        "vrf": "GREEN",
                    },
                ]
            }
        }
        result = verify_evpn_routes(tgen, topo, "e1", input_routes)
        assert result is True, "Testcase {} :Failed \n Error: {}".format(
            tc_name, result
        )

    step(
        "Verify that DCG-2 refresh zebra with EVPN routes. "
        "(no significance of 'rib-stale-time'"
    )

    for addr_type in ADDR_TYPES:
        input_routes = {
            "r4": {
                "static_routes": [
                    {
                        "network": NETWORK1_3[addr_type],
                        "next_hop": NEXT_HOP_IP[addr_type],
                        "vrf": "BLUE",
                    },
                    {
                        "network": NETWORK1_4[addr_type],
                        "next_hop": NEXT_HOP_IP[addr_type],
                        "vrf": "GREEN",
                    },
                ]
            }
        }
        result = verify_rib(tgen, addr_type, "d2", input_routes)
        assert result is True, "Testcase {} :Failed \n Error: {}".format(
            tc_name, result
        )

    write_test_footer(tc_name)


@pytest.mark.parametrize("attribute", ["locPrf", "weight", "path"])
def test_bgp_attributes_for_evpn_address_family_p1(request, attribute):
    """
    BGP attributes for EVPN address-family.
    """

    tgen = get_topogen()
    tc_name = request.node.name
    write_test_header(tc_name)
    check_router_status(tgen)
    reset_config_on_routers(tgen)
    add_default_routes(tgen)

    if tgen.routers_have_failure():
        pytest.skip(tgen.errors)

    step(
        "Advertise prefixes from VNF routers R1 and R2 in associated "
        "VRFs for both address-family."
    )

    for addr_type in ADDR_TYPES:
        input_dict_1 = {
            "r1": {
                "static_routes": [
                    {
                        "network": NETWORK1_1[addr_type],
                        "next_hop": NEXT_HOP_IP[addr_type],
                        "vrf": "RED",
                    }
                ]
            },
            "r2": {
                "static_routes": [
                    {
                        "network": NETWORK2_1[addr_type],
                        "next_hop": NEXT_HOP_IP[addr_type],
                        "vrf": "BLUE",
                    },
                    {
                        "network": NETWORK3_1[addr_type],
                        "next_hop": NEXT_HOP_IP[addr_type],
                        "vrf": "GREEN",
                    },
                ]
            },
        }

        result = create_static_routes(tgen, input_dict_1)
        assert result is True, "Testcase {} : Failed \n Error: {}".format(
            tc_name, result
        )

    topo_local = deepcopy(topo)

    logger.info("Modifying topology b/w e1 and d1 from iBGP to eBGP")
    step("Delete BGP config for vrf RED.")

    if attribute == "locPrf":
        input_dict_vni = {
            "d1": {
                "vrfs": [
                    {"name": "RED", "no_vni": VNI_1},
                    {"name": "BLUE", "no_vni": VNI_2},
                    {"name": "GREEN", "no_vni": VNI_3},
                ]
            }
        }
        result = create_vrf_cfg(tgen, topo, input_dict=input_dict_vni)
        assert result is True, "Testcase {} :Failed \n Error: {}".format(
            tc_name, result
        )

        expected = {"numL3Vnis": 0}
        test_func = functools.partial(
            topotest.router_json_cmp,
            tgen.gears["d1"],
            "show bgp l2vpn evpn vni json",
            expected,
        )
        _, result = topotest.run_and_expect(test_func, None, count=5, wait=3)
        assert result is None, "Testcase {} :Failed \n Error: {}".format(
            tc_name, result
        )

        input_dict_2 = {}
        for dut in ["d1"]:
            temp = {dut: {"bgp": []}}
            input_dict_2.update(temp)

            INDEX = [0, 1, 2, 3]
            VRFS = ["RED", "BLUE", "GREEN", None]
            AS_NUM = [100, 100, 100, 100]

            for index, vrf, as_num in zip(INDEX, VRFS, AS_NUM):
                topo_local["routers"][dut]["bgp"][index]["local_as"] = 200
                if vrf:
                    temp[dut]["bgp"].append(
                        {"local_as": as_num, "vrf": vrf, "delete": True}
                    )
                else:
                    temp[dut]["bgp"].append({"local_as": as_num, "delete": True})

        result = create_router_bgp(tgen, topo, input_dict_2)
        assert result is True, "Testcase {} on d1 :Failed \n Error: {}".format(
            tc_name, result
        )

        result = create_router_bgp(tgen, topo_local["routers"])
        assert result is True, "Testcase {} :Failed \n Error: {}".format(
            tc_name, result
        )

    step("Advertise VRF routes in EVPN address-family from DCG-1 " "and DCG-2 routers.")

    for addr_type in ADDR_TYPES:
        input_dict_1 = {
            "r3": {
                "static_routes": [
                    {
                        "network": NETWORK1_2[addr_type],
                        "next_hop": NEXT_HOP_IP[addr_type],
                        "vrf": "RED",
                    }
                ]
            },
            "r4": {
                "static_routes": [
                    {
                        "network": NETWORK1_3[addr_type],
                        "next_hop": NEXT_HOP_IP[addr_type],
                        "vrf": "BLUE",
                    },
                    {
                        "network": NETWORK1_4[addr_type],
                        "next_hop": NEXT_HOP_IP[addr_type],
                        "vrf": "GREEN",
                    },
                ]
            },
        }

        result = create_static_routes(tgen, input_dict_1)
        assert result is True, "Testcase {} : Failed \n Error: {}".format(
            tc_name, result
        )

    step(
        "Redistribute static in (IPv4 and IPv6) address-family "
        "on Edge-1 for all VRFs."
    )

    input_dict_2 = {}
    for dut in ["r3", "r4"]:
        temp = {dut: {"bgp": []}}
        input_dict_2.update(temp)

        if dut == "r3":
            VRFS = ["RED"]
            AS_NUM = [3]
        if dut == "r4":
            VRFS = ["BLUE", "GREEN"]
            AS_NUM = [4, 4]

        for vrf, as_num in zip(VRFS, AS_NUM):
            temp[dut]["bgp"].append(
                {
                    "local_as": as_num,
                    "vrf": vrf,
                    "address_family": {
                        "ipv4": {
                            "unicast": {"redistribute": [{"redist_type": "static"}]}
                        },
                        "ipv6": {
                            "unicast": {"redistribute": [{"redist_type": "static"}]}
                        },
                    },
                }
            )

    result = create_router_bgp(tgen, topo, input_dict_2)
    assert result is True, "Testcase {} :Failed \n Error: {}".format(tc_name, result)

    step(
        "Verify on router Edge-1 that EVPN routes corresponding to "
        "all VRFs are received from both routers DCG-1 and DCG-2"
    )

    for addr_type in ADDR_TYPES:
        input_routes = {
            "r3": {
                "static_routes": [
                    {
                        "network": NETWORK1_2[addr_type],
                        "next_hop": NEXT_HOP_IP[addr_type],
                        "vrf": "RED",
                    }
                ]
            },
            "r4": {
                "static_routes": [
                    {
                        "network": NETWORK1_3[addr_type],
                        "next_hop": NEXT_HOP_IP[addr_type],
                        "vrf": "BLUE",
                    },
                    {
                        "network": NETWORK1_4[addr_type],
                        "next_hop": NEXT_HOP_IP[addr_type],
                        "vrf": "GREEN",
                    },
                ]
            },
        }

        result = verify_rib(tgen, addr_type, "e1", input_routes)
        assert result is True, "Testcase {} :Failed \n Error: {}".format(
            tc_name, result
        )

    step(
        "Configure a route-map on Edge-1 to modify below BGP attributes "
        "for EVPN address-family:"
    )

    if attribute == "path":
        input_dict_1 = {
            "e1": {
                "route_maps": {
                    "rmap_d1": [
                        {
                            "action": "permit",
                            "set": {
                                attribute: {
                                    "as_num": "123 231 321",
                                    "as_action": "prepend",
                                }
                            },
                        }
                    ],
                    "rmap_d2": [
                        {
                            "action": "permit",
                            "set": {
                                attribute: {"as_num": "121", "as_action": "prepend"}
                            },
                        }
                    ],
                }
            }
        }
    else:
        input_dict_1 = {
            "e1": {
                "route_maps": {
                    "rmap_d1": [{"action": "permit", "set": {attribute: 120}}],
                    "rmap_d2": [{"action": "permit", "set": {attribute: 150}}],
                }
            }
        }
    result = create_route_maps(tgen, input_dict_1)
    assert result is True, "Testcase {} : Failed \n Error: {}".format(tc_name, result)

    input_dict_2 = {
        "e1": {
            "bgp": [
                {
                    "local_as": "100",
                    "address_family": {
                        "l2vpn": {
                            "evpn": {
                                "neighbor": {
                                    "d1": {
                                        "ipv4": {
                                            "e1-link1": {
                                                "route_maps": [
                                                    {
                                                        "name": "rmap_d1",
                                                        "direction": "in",
                                                    }
                                                ]
                                            }
                                        }
                                    },
                                    "d2": {
                                        "ipv4": {
                                            "e1-link1": {
                                                "route_maps": [
                                                    {
                                                        "name": "rmap_d2",
                                                        "direction": "in",
                                                    }
                                                ]
                                            }
                                        }
                                    },
                                }
                            }
                        }
                    },
                }
            ]
        }
    }

    result = create_router_bgp(tgen, topo, input_dict_2)
    assert result is True, "Testcase {} :Failed \n Error: {}".format(tc_name, result)

    step(
        "Verify on router Edge-1 that EVPN routes are preferred via"
        " DCG-1 or DCG-2 based on best path selection criteria "
        "(according to the configured BGP attribute values in route-map)."
    )

    for addr_type in ADDR_TYPES:
        input_routes = {
            "r3": {
                "static_routes": [
                    {
                        "network": NETWORK1_2[addr_type],
                        "next_hop": NEXT_HOP_IP[addr_type],
                        "vrf": "RED",
                    }
                ]
            },
            "r4": {
                "static_routes": [
                    {
                        "network": NETWORK1_3[addr_type],
                        "next_hop": NEXT_HOP_IP[addr_type],
                        "vrf": "BLUE",
                    },
                    {
                        "network": NETWORK1_4[addr_type],
                        "next_hop": NEXT_HOP_IP[addr_type],
                        "vrf": "GREEN",
                    },
                ]
            },
        }

        result = verify_best_path_as_per_bgp_attribute(
            tgen, addr_type, "e1", input_routes, attribute
        )
        assert result is True, "Testcase {} : Failed \n Error: {}".format(
            tc_name, result
        )

    write_test_footer(tc_name)


if __name__ == "__main__":
    args = ["-s"] + sys.argv[1:]
    sys.exit(pytest.main(args))



#!/usr/bin/env python
# SPDX-License-Identifier: ISC

#
# test_bgp_prefix_sid2.py
# Part of NetDEF Topology Tests
#
# Copyright (c) 2020 by LINE Corporation
# Copyright (c) 2020 by Hiroki Shirokura <slank.dev@gmail.com>
#

"""
test_bgp_prefix_sid2.py: Test BGP topology with EBGP on prefix-sid
"""

import json
import os
import sys
import functools
import pytest

CWD = os.path.dirname(os.path.realpath(__file__))
sys.path.append(os.path.join(CWD, "../"))

# pylint: disable=C0413
from lib import topotest
from lib.topogen import Topogen, TopoRouter, get_topogen
from lib.topolog import logger

pytestmark = [pytest.mark.bgpd]


def build_topo(tgen):
    router = tgen.add_router("r1")
    switch = tgen.add_switch("s1")
    switch.add_link(router)

    switch = tgen.gears["s1"]
    peer1 = tgen.add_exabgp_peer("peer1", ip="10.0.0.101", defaultRoute="via 10.0.0.1")
    switch.add_link(peer1)


def setup_module(module):
    tgen = Topogen(build_topo, module.__name__)
    tgen.start_topology()

    router = tgen.gears["r1"]
    router.load_config(
        TopoRouter.RD_ZEBRA, os.path.join(CWD, "{}/zebra.conf".format("r1"))
    )
    router.load_config(
        TopoRouter.RD_BGP, os.path.join(CWD, "{}/bgpd.conf".format("r1"))
    )
    router.start()

    logger.info("starting exaBGP")
    peer_list = tgen.exabgp_peers()
    for pname, peer in peer_list.items():
        logger.info("starting exaBGP on {}".format(pname))
        peer_dir = os.path.join(CWD, pname)
        env_file = os.path.join(CWD, pname, "exabgp.env")
        logger.info("Running ExaBGP peer on {}".format(pname))
        peer.start(peer_dir, env_file)
        logger.info(pname)


def teardown_module(module):
    tgen = get_topogen()
    tgen.stop_topology()


def open_json_file(filename):
    try:
        with open(filename, "r") as f:
            return json.load(f)
    except IOError:
        assert False, "Could not read file {}".format(filename)


def test_r1_rib():
    def _check(name, cmd, expected_file):
        logger.info("polling")
        tgen = get_topogen()
        router = tgen.gears[name]
        output = json.loads(router.vtysh_cmd(cmd))
        expected = open_json_file("{}/{}".format(CWD, expected_file))
        return topotest.json_cmp(output, expected)

    def check(name, cmd, expected_file):
        logger.info('[+] check {} "{}" {}'.format(name, cmd, expected_file))
        tgen = get_topogen()
        func = functools.partial(_check, name, cmd, expected_file)
        _, result = topotest.run_and_expect(func, None, count=10, wait=0.5)
        assert result is None, "Failed"

    check("r1", "show bgp ipv6 vpn 2001:1::/64 json", "r1/vpnv6_rib_entry1.json")
    check("r1", "show bgp ipv6 vpn 2001:2::/64 json", "r1/vpnv6_rib_entry2.json")


if __name__ == "__main__":
    args = ["-s"] + sys.argv[1:]
    ret = pytest.main(args)
    sys.exit(ret)
